@@ -8,7 +8,7 @@ W 0 0_2; right""")
 
 b = a.noisy()
 
-Vn = b.C.V.n(f).subs({'R':10e3, 'C':1e-9, 'T':293, 'k':1.38e-23})   
+Vn = b.C.V.n(f).subs({'R':10e3, 'C':1e-9, 'T':293, 'k_b':1.38e-23})   
 
 from numpy import linspace
 vf = linspace(0, 100e3, 200)
