@@ -42,10 +42,7 @@ from .voltage import *
 from .current import *
 from .admittance import *
 from .impedance import *
-from .resistance import *
-from .reactance import *
-from .conductance import *
-from .susceptance import *
+from .transfer import *
 from .printing import *
 from .sym import *
 from .matrix import *
@@ -62,7 +59,10 @@ from .simulator import *
 from .fexpr import fexpr
 from .sexpr import sexpr, zp2tf, tf, pr2tf
 from .texpr import texpr
+from .cexpr import cexpr
 from .omegaexpr import omegaexpr
+from .phasor import phasor
+from .discretetime import *
 
 def show_version():
     """Show versions of Lcapy, SymPy, NumPy, MatplotLib, SciPy, and Python."""
