@@ -183,4 +183,90 @@ class CircuitGraph(nx.Graph):
 
         return cloops
     
-# neighbors(node) gives neighbouring nodes
+    @property
+    def components(self):
+        """Return list of component names."""
+
+        return [d['name'] for n1, n2, d in self.edges(data=True)]
+
+    def in_series(self, cpt_name):
+        """Return set of component names in series with cpt including itself."""        
+
+        cct = self.cct
+        elt = cct.elements[cpt_name]
+        nodenames = [cct.node_map[nodename] for nodename in elt.nodenames]
+
+        series = []
+        series.append(cpt_name)        
+
+        def follow(node):
+            neighbours = self[node]
+            if len(neighbours) > 2:
+                return
+            for n, e in neighbours.items():
+                if not e['name'] in series:
+                    series.append(e['name'])
+                    follow(n)
+
+        follow(nodenames[0])
+        follow(nodenames[1])        
+
+        # If only have two components in parallel, they will be
+        # detected as a series connection.  However, if there is a
+        # dummy wire, the components are in parallel.
+        for name in series:
+            if name.startswith('W'):
+                return set((cpt_name, ))
+
+        return set(series)
+
+    def in_parallel(self, cpt_name):
+        """Return set of component names in parallel with cpt including itself."""        
+
+        cct = self.cct
+        elt = cct.elements[cpt_name]
+        nodenames = [cct.node_map[nodename] for nodename in elt.nodenames]
+
+        n1, n2 = nodenames[0:2]
+
+        # This is trivial for a multigraph but a mutigraph adds additional problems
+        # since component() will fail if have multiple edges between the same nodes.
+        # edges = self.get_edge_data(n1, n2)
+        # parallel = [d['name'] for k, d in edges.items()]
+
+        parallel = [cpt_name]
+
+        neighbours1 = self[n1]
+        neighbours2 = self[n2]
+
+        # The first created parallel component has no dummy nodes.
+        try:
+            name = self.get_edge_data(n1, n2)['name']
+            parallel.append(name)
+        except:
+            pass
+            
+        # If find a dummy node name then there is a parallel component.
+        
+        for n, e in neighbours1.items():
+            if n.startswith('*'):
+                for n3, e3 in self[n].items():
+                    if n3 == n2 and not e['name'].startswith('W'):
+                        parallel.append(e['name'])
+        for n, e in neighbours2.items():
+            if n.startswith('*'):
+                for n3, e3 in self[n].items():
+                    if n3 == n1 and not e['name'].startswith('W'):
+                        parallel.append(e['name'])
+
+        if n1.startswith('*'):
+            for n3, e3 in self[n1].items():
+                if n3 == n2 and not e['name'].startswith('W'):
+                    parallel.append(e['name'])
+        if n2.startswith('*'):
+            for n3, e3 in self[n2].items():
+                if n3 == n1 and not e['name'].startswith('W'):
+                    parallel.append(e['name'])                                            
+        
+        return set(parallel)
+
