@@ -30,11 +30,11 @@ Dzener: Dname Np Nm zener; Zener diode
 Dphoto: Dname Np Nm photo; Photo diode
 Dtunnel: Dname Np Nm tunnel; Tunnel diode
 Dschottky: Dname Np Nm schottky; Schottky diode
-E: Ename Np Nm Ncp Ncm [Value]; Voltage controlled voltage source
+E: Ename Np Nm Ncp Ncm [Value] [Ac]; Voltage controlled voltage source
 VCVS: VCVSname Np Nm Ncp Ncm [Value]; Voltage controlled voltage source
-Eopamp: Ename Np Nm opamp Ncp Ncm [Value]; Opamp
-Efdopamp: Ename Np Nm fdopamp Ncp Ncm [Value]; Fully differential opamp
-Eamp: Ename Np Nm amp Ncp Ncm [Value]; Amplifier
+Eopamp: Ename Np Nm opamp Ncp Ncm [Value] [Ac]; Opamp
+Efdopamp: Ename Np Nm fdopamp Ncp Ncm [Value] [Ac]; Fully differential opamp
+Eamp: Ename Np Nm amp Ncp Ncm [Value] [Ac]; Amplifier
 F: Fname Np Nm Vcontrol [Value]; Current controlled current source (note the control current is specified through a voltage source)
 CCCS: CCCSname Np Nm Vcontrol [Value]; Current controlled current source (note the control current is specified through a voltage source)
 FB: FBname Np Nm; Ferrite bead
@@ -234,4 +234,5 @@ Power: value; Power
 Lname1: name; Inductor1 name 
 Lname2: name; Inductor2 name 
 Vcontrol: name; Control voltage name 
+Ac: value; Common-mode gain
 """
