@@ -1,5 +1,5 @@
 """This module provides support for Laplace transforms.  It acts as a
-wrapper for SymPy's Laplace transform.  It calculates the unilateral
+quantity for SymPy's Laplace transform.  It calculates the unilateral
 Laplace transform using:
 
 F(s) = lim_{t_0\rightarrow 0} \int_{-t_0}^{\infty} f(t) e^{-s t} dt
@@ -164,6 +164,10 @@ def laplace_sin_cos(expr, t, s):
 
 def laplace_term(expr, t, s):
 
+    # Unilateral LT ignores expr for t < 0 so remove Piecewise.
+    if expr.is_Piecewise and expr.args[0].args[1].has(t >= 0):
+        expr = expr.args[0].args[0]
+    
     const, expr = factor_const(expr, t)
 
     tsym = sympify(str(t))
@@ -241,9 +245,9 @@ def laplace_transform(expr, t, s, evaluate=True):
     else:
         expr = sympify(expr)
 
-    # SymPy laplace barfs on Piecewise but unilateral LT ignores expr
-    # for t < 0 so remove Piecewise.
     expr = expr.replace(var, t)        
+        
+    # Unilateral LT ignores expr for t < 0 so remove Piecewise.
     if expr.is_Piecewise and expr.args[0].args[1].has(t >= 0):
         expr = expr.args[0].args[0]
 
@@ -590,14 +594,24 @@ def inverse_laplace_make(t, const, cresult, uresult, **assumptions):
     elif assumptions.get('ac', False):
 
         if cresult != 0:
-            raise ValueError('Inverse laplace transform weirdness for %s'
-                             ' with is_ac True' % result)
+            # Quietly drop DiracDelta term.  This will appear when
+            # trying to determine the current through a capacitor
+            # when a cosinusoidal voltage source is applied.
+            if cresult.has(sym.DiracDelta):
+                result = uresult
+            else:
+                raise ValueError('Inverse laplace transform weirdness for %s'
+                                 ' with is_ac True' % result)
+            
         # TODO, perform more checking of the result.
         
     elif not assumptions.get('causal', False):
 
-        # Cannot determine result for t < 0        
-        result = sym.Piecewise((result, t >= 0))
+        if uresult != 0:
+            # Cannot determine result for t < 0
+            result = sym.Piecewise((const * (uresult + cresult), t >= 0))
+        else:
+            result = const * cresult
         
     return result
 
