@@ -1,5 +1,5 @@
 """
-This module implements a wrapper for the SymPy Matrix class.
+This module implements a quantity for the SymPy Matrix class.
 
 Copyright 2019--2020 Michael Hayes, UCECE
 """
@@ -32,10 +32,8 @@ class Matrix(sym.Matrix):
 
     # What's worse, is that calling _sympify on each element creates
     # different variables than what we are expecting.  For example,
-    # the sExpr s looks the same but gets different attributes.  We
-    # prevent this by defining _sympify.
-    
-    # _typewrap = sExpr
+    # the LaplaceDomainExpression s looks the same but gets different
+    # attributes.  We prevent this by defining _sympify.
 
     _sympify = staticmethod(msympify)
     
@@ -64,7 +62,6 @@ class Matrix(sym.Matrix):
         LaTeX markup.  However, this requires mathjax.  If this method
         is not defined, jupyter falls back on _repr__pretty_ which
         outputs unicode."""
-        import pdb; pdb.set_trace()
         return '$$' + latex(self) + '$$'
 
     def pprint(self):
@@ -95,7 +92,8 @@ class Matrix(sym.Matrix):
 
         return expr(super(Matrix, self).norm())
 
-    # TODO, either need to explicitly wrap methods or use some cunning implicit method.
+    # TODO, either need to explicitly wrap methods or use some cunning
+    # implicit method.
 
     def replace(self, query, value, map=False, simultaneous=True, exact=None):
 
@@ -140,8 +138,8 @@ def matrix(mat):
     """
 
     from .sym import tsym, ssym
-    from .smatrix import sMatrix
-    from .tmatrix import tMatrix
+    from .smatrix import LaplaceDomainMatrix
+    from .tmatrix import TimeDomainMatrix
     
     elt = mat[0]
     try:
@@ -150,9 +148,9 @@ def matrix(mat):
         pass    
     
     if elt.has(tsym):
-        return tMatrix(mat)
+        return TimeDomainMatrix(mat)
     elif elt.has(ssym):
-        return sMatrix(mat)        
+        return LaplaceDomainMatrix(mat)        
     else:
         return mat
 
