@@ -8,14 +8,14 @@ Copyright 2015--2020 Michael Hayes, UCECE
 
 from __future__ import print_function
 from .expr import expr
-from .cexpr import cExpr
-from .omegaexpr import omegaExpr
+from .cexpr import ConstantExpression
+from .omegaexpr import AngularFourierDomainExpression
 from .symbols import j, omega, jomega, s, t
 from .functions import sqrt
 from .sym import capitalize_name, omegasym
 from .grammar import delimiters
-from .immittancemixin import ImmittanceMixin
-from .current import Current
+from .immittancemixinhack import ImmittanceMixinHack
+from .superpositioncurrent import SuperpositionCurrent
 from .opts import Opts
 import lcapy
 import inspect
@@ -30,7 +30,7 @@ cptaliases = {'E': 'VCVS', 'F': 'CCCS',
               'k': 'Spring'}
 
 
-class Cpt(ImmittanceMixin):
+class Cpt(ImmittanceMixinHack):
 
     dependent_source = False
     independent_source = False    
@@ -327,7 +327,9 @@ class Cpt(ImmittanceMixin):
         """Return True if causal component or if source produces
         a causal signal."""
 
-        return self._source_IV.is_causal
+        if self.is_source:
+            return self._source_IV.is_causal
+        return self.cpt.is_causal
 
     @property
     def is_dc(self):
@@ -500,17 +502,27 @@ class Cpt(ImmittanceMixin):
     
     @property
     def admittance(self):
-        """Self admittance of component.  For the driving-point
-        admittance measured across the component use .dpY or .oneport().Y"""
+        """Self admittance of component. 
 
-        return self.cpt.admittance.new(self.cct.kind)        
+        The admittance is expressed in jomega form for AC circuits
+        and in s-domain for for transient circuits.
+
+        For the driving-point admittance measured across the component
+        use .dpY or .oneport().Y"""
+
+        return self.cpt.admittance._select(self.cct.kind)        
 
     @property
     def impedance(self):
-        """Self impedance of component.  For the driving-point impedance
-        measured across the component use .dpZ or .oneport().Z"""        
+        """Self impedance of component.  
 
-        return self.cpt.impedance.new(self.cct.kind)
+        The impedance is expressed in jomega form for AC circuits
+        and in s-domain for for transient circuits.
+
+        For the driving-point impedance measured across the component
+        use .dpZ or .oneport().Z"""        
+
+        return self.cpt.impedance._select(self.cct.kind)
 
     @property
     def dpIsc(self):
@@ -757,7 +769,7 @@ class RLC(Cpt):
         for opt, val in voltage_opts.items():
             opts[opt] = capitalize_name(val)
 
-        onet = self._netmake_variant('O', opts=opts)
+        onet = self._netmake_O(opts=opts)
         return znet + '\n' + vnet + '\n' + onet
 
 
@@ -771,8 +783,10 @@ class RC(RLC):
 
         rnet = self._netmake(nodes=(self.relnodes[0], dummy_node),
                              opts=opts)                                  
-        
-        Vn = 'sqrt(4 * k * T * %s)' % self.args[0]
+
+        # Use k_b for Boltzmann's constant to avoid clash with k symbol
+        # for discrete frequency
+        Vn = 'sqrt(4 * k_b * T * %s)' % self.args[0]
         vnet = self._netmake_variant('Vn', nodes=(dummy_node, self.relnodes[1]),
                                      args=('noise', Vn), opts=opts)
         return rnet + '\n' + vnet
@@ -889,9 +903,9 @@ class VCVS(DependentSource):
             cct._B[n2, m] -= 1
             cct._C[m, n2] -= 1
 
-        Ad = cExpr(self.args[0]).expr
+        Ad = ConstantExpression(self.args[0]).expr
         if len(self.args) > 1:
-            Ac = cExpr(self.args[1]).expr
+            Ac = ConstantExpression(self.args[1]).expr
         else:
             Ac = 0
 
@@ -919,7 +933,7 @@ class CCCS(DependentSource):
     def _stamp(self, cct):
         n1, n2 = self.node_indexes
         m = cct._branch_index(self.args[0])
-        F = cExpr(self.args[1]).expr
+        F = ConstantExpression(self.args[1]).expr
             
         if n1 >= 0:
             cct._B[n1, m] -= F
@@ -944,7 +958,7 @@ class VCCS(DependentSource):
 
     def _stamp(self, cct):
         n1, n2, n3, n4 = self.node_indexes
-        G = cExpr(self.args[0]).expr
+        G = ConstantExpression(self.args[0]).expr
 
         if n1 >= 0 and n3 >= 0:
             cct._G[n1, n3] -= G
@@ -986,7 +1000,7 @@ class GY(Dummy):
         # V2 = -I1 Z2     V1 = I2 Z1
         # where V2 = V[n1] - V[n2] and V1 = V[n3] - V[n4]
         
-        Z1 = cExpr(self.args[0]).expr                    
+        Z1 = ConstantExpression(self.args[0]).expr                    
         Z2 = Z1
         
         if n1 >= 0:
@@ -1024,7 +1038,7 @@ class CCVS(DependentSource):
             cct._C[m, n2] -= 1
         
         mc = cct._branch_index(self.args[0])
-        G = cExpr(self.args[1]).expr
+        G = ConstantExpression(self.args[1]).expr
         cct._D[m, mc] -= G
 
     def _kill(self):
@@ -1099,7 +1113,6 @@ class K(Dummy):
 
         if cct.kind in ('s', 'ivp', 'laplace'):
             # FIXME, generalise for other domains...
-            # The Impedance class really needs fixing.
             ZM = K.expr * sym.sqrt(ZL1 * ZL2 / s**2) * s
         else:
             ZM = K.expr * sym.sqrt(ZL1 * ZL2)
@@ -1205,11 +1218,11 @@ class O(Dummy):
 
     @property
     def I(self):
-        return Current(0)
+        return SuperpositionCurrent(0)
 
     @property
     def i(self):
-        return Current(0)(t)
+        return SuperpositionCurrent(0)(t)
     
 
 class P(O):
@@ -1394,7 +1407,7 @@ class TR(Dummy):
             cct._B[n2, m] += 1
             cct._C[m, n2] += 1
         
-        A = cExpr(self.args[0]).expr
+        A = ConstantExpression(self.args[0]).expr
         
         if n1 >= 0:
             cct._C[m, n1] -= A
