@@ -6,10 +6,9 @@ Copyright 2020 Michael Hayes, UCECE
 """
 
 from .expr import expr
-from .sexpr import Hs, Zs, Ys
 from .mnacpts import Cpt
-from .impedance import Impedance
-from .admittance import Admittance
+from .impedance import impedance
+from .admittance import admittance
 from .equipotentialnodes import EquipotentialNodes
 from .node import Node
 from .state import state
@@ -19,7 +18,7 @@ from .attrdict import AttrDict
 from .netfile import NetfileMixin
 from .statespace import StateSpace
 from .voltage import Vname
-from .current import Iname
+from .current import Iname, current
 from .simulator import Simulator
 from .netlistnamespace import NetlistNamespace
 from .matrix import Matrix
@@ -538,7 +537,7 @@ class NetlistMixin(object):
         If = new.Vin_.I
         new.remove('Vin_')
 
-        return Admittance(If.laplace(), kind=self.kind)
+        return admittance(If.laplace().expr)
 
     def impedance(self, Np, Nm=None):
         """Return driving-point impedance between nodes
@@ -560,7 +559,7 @@ class NetlistMixin(object):
         Vf = new.Voc(Np, Nm)
         new.remove('Iin_')
 
-        return Impedance(Vf.laplace(), kind=self.kind)
+        return impedance(Vf.laplace().expr)
 
     def resistance(self, Np, Nm=None):
         """Return resistance between nodes Np and Nm with independent
@@ -606,7 +605,9 @@ class NetlistMixin(object):
         V2 = new.Voc(N2p, N2m)
         V1 = new.V1_.V
 
-        return Hs(V2.laplace() / V1.laplace(), causal=True)
+        H = V2.laplace() / V1.laplace()
+        H.causal = True
+        return H
 
     def Aparams(self, N1p, N1m, N2p, N2m):
         """Create A-parameters for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
@@ -630,11 +631,11 @@ class NetlistMixin(object):
 
             # A11 = V1 / V2 with I2 = 0
             # Apply V1 and measure V2 with port 2 open-circuit
-            A11 = Hs(net.V1_.V(s) / net.Voc(N2p, N2m)(s))
+            A11 = net.V1_.V(s) / net.Voc(N2p, N2m)(s)
 
             # A12 = V1 / I2 with V2 = 0
             # Apply V1 and measure I2 with port 2 short-circuit
-            A12 = Zs(net.V1_.V(s) / net.Isc(N2p, N2m)(s))
+            A12 = net.V1_.V(s) / net.Isc(N2p, N2m)(s)
 
             net.remove('V1_')
 
@@ -643,17 +644,17 @@ class NetlistMixin(object):
             # A21 = I1 / V2 with I2 = 0
             # Apply I1 and measure V2 with port 2 open-circuit
             try:
-                A21 = Ys(1 / net.Voc(N2p, N2m)(s))                
+                A21 = current(0 * s + 1) / net.Voc(N2p, N2m)(s)
             except ValueError:
                 # It is likely there is an open-circuit.                
                 net2 = net.copy()
                 net2.add('W %s %s' % (N2p, N2m))
-                A21 = Ys(-net2.I1_.I(s) / net2.Voc(N2p, N2m)(s))
+                A21 = -net2.I1_.I(s) / net2.Voc(N2p, N2m)(s)
                 A21 = 0                
 
             # A22 = I1 / I2 with V2 = 0
             # Apply I1 and measure I2 with port 2 short-circuit
-            A22 = Hs(1 / net.Isc(N2p, N2m)(s))
+            A22 = current(0 * s + 1) / net.Isc(N2p, N2m)(s)
 
             net.remove('I1_')
             A = AMatrix(((A11, A12), (A21, A22)))
@@ -751,11 +752,11 @@ class NetlistMixin(object):
 
             # Z11 = V1 / I1 with I2 = 0
             # Apply I1 and measure V1 with port 2 open-circuit
-            Z11 = Zs(net.Voc(N1p, N1m)(s))
+            Z11 = impedance(net.Voc(N1p, N1m)(s))
 
             # Z21 = V2 / I1 with I2 = 0
             # Apply I1 and measure V2 with port 2 open-circuit
-            Z21 = Zs(net.Voc(N2p, N2m)(s))          
+            Z21 = impedance(net.Voc(N2p, N2m)(s))          
 
             net.remove('I1_')
 
@@ -763,11 +764,11 @@ class NetlistMixin(object):
 
             # Z12 = V1 / I2 with I1 = 0
             # Apply I2 and measure V1 with port 1 open-circuit
-            Z12 = Zs(net.Voc(N1p, N1m)(s))
+            Z12 = impedance(net.Voc(N1p, N1m)(s))
 
             # Z22 = V2 / I2 with I1 = 0
             # Apply I2 and measure V2 with port 1 open-circuit
-            Z22 = Zs(net.Voc(N2p, N2m)(s))          
+            Z22 = impedance(net.Voc(N2p, N2m)(s))          
 
             net.remove('I2_')            
 
@@ -808,7 +809,7 @@ class NetlistMixin(object):
                         net.add('V%d_ %s %s 0' % (row, ports[row][0], ports[row][1]))                        
 
                 for row in range(len(ports)):
-                    Y[row, col] = Ys(net.elements['V%d_' % row].I(s))
+                    Y[row, col] = admittance(net.elements['V%d_' % row].I(s))
 
                 for row in range(len(ports)):                        
                     net.remove('V%d_' % row)
@@ -860,7 +861,7 @@ class NetlistMixin(object):
                 net.add('I_ %s %s {DiracDelta(t)}' % (ports[col][0], ports[col][1]))
 
                 for row in range(len(ports)):                
-                    Z[row, col] = Zs(net.Voc(ports[row][0], ports[row][1])(s))
+                    Z[row, col] = impedance(net.Voc(ports[row][0], ports[row][1])(s))
 
                 net.remove('I_')
             return Z
@@ -1610,18 +1611,23 @@ class NetlistMixin(object):
         """Return dictionary of source groups.  Each group is a list of
         sourcenames that can be analysed at the same time.  Noise
         sources have separate groups since they are assumed to be
-        uncorrelated.
+        uncorrelated.  The source groups are keyed by the names of
+        superposition decomposition keys: 'dc', 's', 't', 'n*', and omega,
+        where omega is an expression for the angular frequency of a phasor,
+        and `n*` is a noise identifier.
+        
+        If transform is False, the returned keys are 'dc', 't', 's', and 'n*'.
 
-        If transform is True, the source values are decomposed into
-        the different transform domains to determine which domains are
-        required.  In this case, a source can appear in multiple
-        groups.  For example, if a voltage source V1 has a value 10 +
-        5 * cos(omega * t), V1 will be added to the dc and ac groups.
+        If transform is True, the returned keys are 'dc', 's', omega,
+        and 'n*'.  Note, there is no time-domain component.  Note,
+        after transformation, a source can appear in multiple groups.
+        For example, if a voltage source V1 has a value 10 + 5 *
+        cos(omega * t), V1 will be added to the dc and omega groups.
 
         """
 
         groups = {}
-        for key, elt in self.elements.items():
+        for eltname, elt in self.elements.items():
             if not elt.independent_source:
                 continue
             cpt = elt.cpt
@@ -1634,11 +1640,12 @@ class NetlistMixin(object):
                 Isc = cpt.Isc
                 if transform:
                     Isc = Isc.decompose()                
-                cpt_kinds = Isc.keys()                
+                cpt_kinds = Isc.keys()
+                
             for cpt_kind in cpt_kinds:
                 if cpt_kind not in groups:
                     groups[cpt_kind] = []
-                groups[cpt_kind].append(key)
+                groups[cpt_kind].append(eltname)
 
         return groups    
 
@@ -1672,17 +1679,17 @@ class NetlistMixin(object):
         reactances = []
         ics = []
         
-        for key, elt in self.elements.items():
+        for eltname, elt in self.elements.items():
             if elt.need_control_current:
                 control_sources.append(elt.args[0])
             if elt.has_ic is not None:
                 if elt.has_ic:
                     has_ic = True
-                    ics.append(key)
+                    ics.append(eltname)
                 if not elt.zeroic:
                     zeroic = False
             if elt.independent_source:
-                independent_sources.append(key)
+                independent_sources.append(eltname)
                 if elt.has_s_transient:
                     has_s = True
                 if elt.has_transient:
@@ -1694,10 +1701,10 @@ class NetlistMixin(object):
                 if not elt.is_causal:
                     causal = False
             if elt.dependent_source:
-                dependent_sources.append(key)
+                dependent_sources.append(eltname)
             if elt.reactive:
                 reactive = True
-                reactances.append(key)                
+                reactances.append(eltname)                
 
         num_sources = len(independent_sources)
                     
