@@ -1,40 +1,36 @@
-"""This module provides the nExpr class to represent discrete-time expressions.
+"""This module provides the DiscreteTimeDomainExpression class to
+represent discrete-time expressions.
 
-Copyright 2020 Michael Hayes, UCECE
+Copyright 2020--2021 Michael Hayes, UCECE
 
 """
 
 from __future__ import division
-from .seqexpr import seqExpr
+from .domains import DiscreteTimeDomain
+from .seqexpr import SequenceExpression
 from .sequence import Sequence
 from .functions import exp
 from .sym import j, oo, pi, fsym
 from .dsym import nsym, ksym, zsym, dt
-from .acdc import ACChecker, is_dc, is_ac, is_causal
 from .ztransform import ztransform
 from .dft import DFT
 from sympy import Sum, summation, limit
 
 
-__all__ = ('Hn', 'In', 'Vn', 'Yn', 'Zn')
+__all__ = ('nexpr', )
 
-
-class nExpr(seqExpr):
-
-    """t-domain expression or symbol."""
+class DiscreteTimeDomainExpression(DiscreteTimeDomain, SequenceExpression):
+    """Discrete time expression or symbol."""
 
     var = nsym
-    domain_name = 'Sample'
-    domain_units = ''
+
 
     def __init__(self, val, **assumptions):
 
         check = assumptions.pop('check', True)
+        assumptions['real'] = True        
         
-        super(nExpr, self).__init__(val, **assumptions)
-
-        self._discrete_fourier_conjugate_class = kExpr
-        self._ztransform_conjugate_class = zExpr
+        super(DiscreteTimeDomainExpression, self).__init__(val, **assumptions)
 
         expr = self.expr
         if check and expr.find(zsym) != set() and not expr.has(Sum):
@@ -44,34 +40,24 @@ class nExpr(seqExpr):
             raise ValueError(
                 'n-domain expression %s cannot depend on k' % expr)            
 
-    def infer_assumptions(self):
+    def _mul_compatible(self, x):
 
-        self.assumptions['dc'] = False
-        self.assumptions['ac'] = False
-        self.assumptions['causal'] = False
-
-        var = self.var
-        if is_dc(self, var):
-            self.assumptions['dc'] = True
-            return
-
-        if is_ac(self, var):
-            self.assumptions['ac'] = True
-            return
-
-        if is_causal(self, var):
-            self.assumptions['causal'] = True
-
-    def merge_assumptions(self, **assumptions):
+        if x.is_constant_domain:
+            return True
+        if self.quantity == 'undefined' or x.quantity == 'undefined':
+            return True
         
-        new_assumptions = self.assumptions.copy()
-        new_assumptions.update(assumptions)
-        return new_assumptions
+        # TODO: allow TimeDomainVoltage**2 one day.
+        return False
+
+    def as_expr(self):
+        return DiscreteTimeDomainExpression(self)
 
     def differentiate(self):
         """First order difference."""
 
-        return self.__class__((self.expr - self.subs(n - 1).expr) / dt, **self.assumptions)
+        result = (self.expr - self.subs(n - 1).expr) / dt
+        return self.__class__(result, **self.assumptions)
 
     def integrate(self):
         """First order integration."""
@@ -94,17 +80,9 @@ class nExpr(seqExpr):
     def ztransform(self, evaluate=True, **assumptions):
         """Determine one-sided z-transform."""
 
-        self.infer_assumptions()
-
-        assumptions = self.merge_assumptions(**assumptions)
-
+        assumptions = self.assumptions.merge_and_infer(self, **assumptions)
         result = ztransform(self.expr, self.var, zsym, evaluate)
-
-        if hasattr(self, '_ztransform_conjugate_class'):
-            result = self._ztransform_conjugate_class(result, **assumptions)
-        else:
-            result = zExpr(result, **assumptions)
-        return result
+        return self.change(result, domain='Z', **assumptions)
 
     def ZT(self, **assumptions):
         return self.ztransform(**assumptions)
@@ -151,13 +129,7 @@ class nExpr(seqExpr):
             N = sympify('N')
 
         result = DFT(self.expr, nsym, ksym, N, evaluate=evaluate)
-
-        if hasattr(self, '_discrete_fourier_conjugate_class'):
-            result = self._discrete_fourier_conjugate_class(result)
-        else:
-            result = kExpr(result, check=False)
-            
-        return result
+        return self.change(result, domain='discrete fourier')
     
     def delay(self,m):
         """Delay signal by m samples."""
@@ -178,11 +150,8 @@ class nExpr(seqExpr):
     def discrete_time_fourier_transform(self, **assumptions):
         """Convert to Fourier domain using discrete time Fourier transform."""
 
-        self.infer_assumptions()
-
-        assumptions = self.merge_assumptions(**assumptions)
-        
-        if assumptions.get('causal', False):        
+        assumptions = self.assumptions.merge_and_infer(self, **assumptions)
+        if assumptions.is_causal:
             return self.ZT(**assumptions).DTFT()
 
         from .fexpr import fexpr
@@ -204,72 +173,6 @@ class nExpr(seqExpr):
         H = self.ZT()
         return H.difference_equation(input, output, form)
 
-    
-class Yn(nExpr):
-
-    """t-domain 'admittance' value."""
-
-    units = 'siemens/s'
-
-    def __init__(self, val, **assumptions):
-
-        super(Yn, self).__init__(val, **assumptions)
-        self._ztransform_conjugate_class = Yz
-        self._discrete_fourier_conjugate_class = Yk
-
-
-class Zn(nExpr):
-
-    """t-domain 'impedance' value."""
-
-    units = 'ohms/s'
-
-    def __init__(self, val, **assumptions):
-
-        super(Zn, self).__init__(val, **assumptions)
-        self._ztransform_conjugate_class = Zz
-        self._discrete_fourier_conjugate_class = Zk
-
-
-class Vn(nExpr):
-
-    """t-domain voltage (units V)."""
-
-    quantity = 'Voltage'
-    units = 'V'
-
-    def __init__(self, val, **assumptions):
-
-        super(Vn, self).__init__(val, **assumptions)
-        self._ztransform_conjugate_class = Vz
-        self._discrete_fourier_conjugate_class = Vk
-
-class In(nExpr):
-
-    """t-domain current (units A)."""
-
-    quantity = 'Current'
-    units = 'A'
-
-    def __init__(self, val, **assumptions):
-
-        super(In, self).__init__(val, **assumptions)
-        self._ztransform_conjugate_class = Iz
-        self._discrete_fourier_conjugate_class = Ik
-
-
-class Hn(nExpr):
-
-    """impulse response"""
-
-    quantity = 'Impulse response'
-    units = ''
-
-    def __init__(self, val, **assumptions):
-
-        super(Hn, self).__init__(val, **assumptions)
-        self._ztransform_conjugate_class = Hz
-        self._discrete_fourier_conjugate_class = Hk
 
 def nexpr(arg, **assumptions):
     """Create nExpr object.  If `arg` is nsym return n"""
@@ -287,8 +190,15 @@ def nexpr(arg, **assumptions):
     if isinstance(arg, (list, ndarray)):
         return Sequence(arg, var=n).as_impulses()
 
-    return nExpr(arg, **assumptions)
+    return DiscreteTimeDomainExpression(arg, **assumptions)
 
-from .zexpr import Hz, Iz, Vz, Yz, Zz, zExpr
-from .kexpr import Hk, Ik, Vk, Yk, Zk, kExpr
-n = nExpr('n')
+
+from .expressionclasses import expressionclasses
+
+classes = expressionclasses.make(DiscreteTimeDomainExpression)
+expressionclasses.add('discrete time', classes)
+
+from .zexpr import ZDomainExpression
+from .kexpr import DiscreteFourierDomainExpression
+
+n = DiscreteTimeDomainExpression('n')
