@@ -6,7 +6,7 @@ Copyright 2019-2020 Michael Hayes, UCECE
 """
 
 from .circuitgraph import CircuitGraph
-from .tmatrix import tMatrix
+from .tmatrix import TimeDomainMatrix
 from .utils import equation
 from .systemequations import SystemEquations
 import sympy as sym
@@ -111,7 +111,7 @@ class NodalAnalysis(object):
                 lhs, rhs = self._unknowns[n1], self._unknowns[n2] + V
 
             else:
-                result = Current(0).select(self.kind)
+                result = Itype(self.kind)(0)
                 for elt in self.G.connected(node):
                     if len(elt.nodenames) < 2:
                         raise ValueError('Elt %s has too few nodes' % elt)
@@ -207,7 +207,7 @@ class NodalAnalysis(object):
         return self._sys.format(form, invert)
 
 from .expr import ExprDict, expr
-from .texpr import Vt
+from .texpr import TimeDomainVoltage
 from .voltage import Vname
-from .current import Current
+from .current import Itype
 from .matrix import matrix
