@@ -15,24 +15,17 @@ currents, respectively, to model transient responses.
 One-ports can either be connected in series (+) or parallel (|) to
 create a new one-port.
 
-Copyright 2014--2020 Michael Hayes, UCECE
+Copyright 2014--2021 Michael Hayes, UCECE
 """
 
-# TODO.   Rethink best way to handle impedances.   These can either be
-# s-domain or omega domain (or DC as a special case where omega = 0).
-# Perhaps alays use Current and Voltage and have these classes
-# handle multiplication/division with admittances/impedances?
-
-
-
 from __future__ import division
 from .functions import Heaviside, cos, exp
 from .sym import omega0sym, tsym, oo
 from .symbols import j, t, s
 from .network import Network
 from .immittancemixin import ImmittanceMixin
-from .impedance import Impedance
-from .admittance import Admittance
+from .impedance import impedance
+from .admittance import admittance
 from sympy import Derivative, Integral
 
 
@@ -86,18 +79,18 @@ class OnePort(Network, ImmittanceMixin):
         if self._Z is not None:
             return self._Z
         if self._Y is not None:
-            return Impedance(1 / self._Y)
+            return 1 / self._Y
         if self._Voc is not None:        
-            return Impedance(0)
+            return impedance(0)
         if self._Isc is not None:        
-            return Impedance(1 / Admittance(0))
+            return 1 / admittance(0)
         raise ValueError('_Isc, _Voc, _Y, or _Z undefined for %s' % self)
 
     @property
     def admittance(self):
         if self._Y is not None:
             return self._Y
-        return Admittance(1 / self.impedance)
+        return 1 / self.impedance
 
     @property
     def Voc(self):
@@ -105,11 +98,11 @@ class OnePort(Network, ImmittanceMixin):
         if self._Voc is not None:
             return self._Voc
         if self._Isc is not None:
-            return self._Isc * self.impedance
+            return self._Isc._mul(self.impedance)
         if self._Z is not None:        
-            return Voltage(0)
+            return SuperpositionVoltage(0)
         if self._Y is not None:        
-            return Current(0)
+            return SuperpositionCurrent(0)
         raise ValueError('_Isc, _Voc, _Y, or _Z undefined for %s' % self)
     
     @property
@@ -117,7 +110,7 @@ class OnePort(Network, ImmittanceMixin):
         """Short-circuit current."""        
         if self._Isc is not None:
             return self._Isc
-        return self.Voc / self.impedance
+        return self.Voc._mul(self.admittance)
 
     @property
     def V(self):
@@ -127,7 +120,7 @@ class OnePort(Network, ImmittanceMixin):
     @property
     def I(self):
         """Open-circuit current.  Except for a current source this is zero."""
-        return Current(0)
+        return SuperpositionCurrent(0)
 
     @property
     def i(self):
@@ -320,7 +313,7 @@ class OnePort(Network, ImmittanceMixin):
         
         R1 = self.R
         if R1 != 0:
-            Vn = Vnoise('sqrt(4 * k * T * %s)' % R1(j * omega))
+            Vn = Vnoise('sqrt(4 * k_b * T * %s)' % R1(j * omega))
             return self + Vn
         return self
 
@@ -690,11 +683,11 @@ class Par(ParSer):
         Y = 0
         for arg in self.args:
             Y += arg.admittance
-        return Admittance(Y)
+        return Y
 
     @property
     def impedance(self):
-        return Impedance(1 / self.admittance)
+        return 1 / self.admittance
 
 class Ser(ParSer):
     """Series class"""
@@ -763,15 +756,15 @@ class Ser(ParSer):
         return False
     
     @property
-    def Admittance(self):
-        return Admittance(1 / self.impedance)
+    def admittance(self):
+        return 1 / self.impedance
     
     @property
     def impedance(self):
         Z = 0
         for arg in self.args:
             Z += arg.impedance
-        return Impedance(Z)
+        return Z
 
     
 class R(OnePort):
@@ -782,16 +775,16 @@ class R(OnePort):
     def __init__(self, Rval):
 
         self.args = (Rval, )
-        self._R = cExpr(Rval)
-        self._Z = Impedance(self._R)
+        self._R = cexpr(Rval)
+        self._Z = impedance(self._R, causal=True)
 
     def i_equation(self, v, kind='t'):
 
-        return Current(Voltage(v).select(kind) / self._Z._selectexpr(kind)).select(kind)
+        return SuperpositionCurrent(SuperpositionVoltage(v).select(kind) / self._Z._selectexpr(kind)).select(kind)
 
     def v_equation(self, i, kind='t'):
 
-        return Voltage(Current(i).select(kind) * self._Z._selectexpr(kind)).select(kind)
+        return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Z._selectexpr(kind)).select(kind)
     
 
 class G(OnePort):
@@ -802,8 +795,8 @@ class G(OnePort):
     def __init__(self, Gval):
 
         self.args = (Gval, )
-        self._G = cExpr(Gval)
-        self._Z = Impedance(1 / self._G)
+        self._G = cexpr(Gval)
+        self._Z = impedance(1 / self._G, causal=True)
 
     def _net_make(self, netlist, n1=None, n2=None, dir='right'):
 
@@ -815,11 +808,11 @@ class G(OnePort):
 
     def i_equation(self, v, kind='t'):
 
-        return Current(Voltage(v).select(kind) / self._Z._selectexpr(kind)).select(kind)
+        return SuperpositionCurrent(SuperpositionVoltage(v).select(kind) / self._Z._selectexpr(kind)).select(kind)
 
     def v_equation(self, i, kind='t'):
 
-        return Voltage(Current(i).select(kind) * self._Z._selectexpr(kind)).select(kind)
+        return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Z._selectexpr(kind)).select(kind)
     
 
 class L(OnePort):
@@ -840,12 +833,12 @@ class L(OnePort):
         else:
             self.args = (Lval, )
 
-        Lval = cExpr(Lval)
-        i0 = cExpr(i0)
+        Lval = cexpr(Lval)
+        i0 = cexpr(i0)
         self.L = Lval
         self.i0 = i0
-        self._Z = Impedance(s * Lval)
-        self._Voc = Voltage(-Vs(i0 * Lval))
+        self._Z = impedance(s * Lval, causal=True)
+        self._Voc = SuperpositionVoltage(LaplaceDomainExpression(-i0 * Lval))
         self.zeroic = self.i0 == 0 
 
     def i_equation(self, v, kind='t'):
@@ -855,14 +848,14 @@ class L(OnePort):
         if kind in ('t', 'time', 'super'):
             u = tausym
             v = expr(v).subs(t, u)
-            return Current(expr(Integral(v.expr, (u, -oo, tsym))) / self.L).select(kind)
-        return Current(Voltage(v).select(kind) / self.Z._selectexpr(kind)).select(kind)
+            return SuperpositionCurrent(expr(Integral(v.expr, (u, -oo, tsym))) / self.L).select(kind)
+        return SuperpositionCurrent(SuperpositionVoltage(v).select(kind) / self.Z._selectexpr(kind)).select(kind)
 
     def v_equation(self, i, kind='t'):
 
         if kind in ('t', 'time', 'super'):
-            return Voltage(self.L * expr(Derivative(i.expr, t))).select(kind)
-        return Voltage(Current(i).select(kind) * self._Z._selectexpr(kind)).select(kind)
+            return SuperpositionVoltage(self.L * expr(Derivative(i.expr, t))).select(kind)
+        return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Z._selectexpr(kind)).select(kind)
 
     
 class C(OnePort):
@@ -883,19 +876,19 @@ class C(OnePort):
         else:
             self.args = (Cval, )
 
-        Cval = cExpr(Cval)
-        v0 = cExpr(v0)
+        Cval = cexpr(Cval)
+        v0 = cexpr(v0)
         self.C = Cval
         self.v0 = v0
-        self._Z = Impedance(1 / (s * Cval))
-        self._Voc = Voltage(Vs(v0) / s)
+        self._Z = impedance(1 / (s * Cval), causal=True)
+        self._Voc = SuperpositionVoltage(LaplaceDomainExpression(v0 / s))
         self.zeroic = self.v0 == 0
 
     def i_equation(self, v, kind='t'):
         
         if kind in ('t', 'time', 'super'):
-            return Current(self.C * expr(Derivative(v.expr, t))).select(kind)
-        return Current(Voltage(v).select(kind) / self._Z._selectexpr(kind)).select(kind)
+            return SuperpositionCurrent(self.C * expr(Derivative(v.expr, t))).select(kind)
+        return SuperpositionCurrent(SuperpositionVoltage(v).select(kind) / self._Z._selectexpr(kind)).select(kind)
 
     def v_equation(self, i, kind='t'):
 
@@ -904,8 +897,8 @@ class C(OnePort):
         if kind in ('t', 'time', 'super'):
             u = tausym
             i = expr(i).subs(t, u)
-            return Voltage(expr(Integral(i.expr, (u, -oo, tsym))) / self.C).select(kind)
-        return Voltage(Current(i).select(kind) * self._Z._selectexpr(kind)).select(kind)
+            return SuperpositionVoltage(expr(Integral(i.expr, (u, -oo, tsym))) / self.C).select(kind)
+        return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Z._selectexpr(kind)).select(kind)
 
 
 class CPE(OnePort):
@@ -927,11 +920,11 @@ class CPE(OnePort):
 
         self.args = (K, alpha)
 
-        K = cExpr(K)
-        alpha = cExpr(alpha)
+        K = cexpr(K)
+        alpha = cexpr(alpha)
         self.K = K
         self.alpha = alpha
-        self._Z = Impedance(1 / (s ** alpha * K))
+        self._Z = impedance(1 / (s ** alpha * K), causal=True)
 
 
 class Y(OnePort):
@@ -940,7 +933,7 @@ class Y(OnePort):
     def __init__(self, Yval):
 
         self.args = (Yval, )
-        Yval = Admittance(Yval)
+        Yval = admittance(Yval)
         self._Y = Yval
 
 
@@ -950,11 +943,11 @@ class Z(OnePort):
     def __init__(self, Zval):
 
         self.args = (Zval, )
-        Zval = Impedance(Zval)
+        Zval = impedance(Zval)
         self._Z = Zval
 
 
-class VoltageSource(OnePort):
+class VoltageSourceBase(OnePort):
 
     voltage_source = True
     netname = 'V'
@@ -962,10 +955,10 @@ class VoltageSource(OnePort):
 
     def v_equation(self, i, kind='t'):
         
-        return Voltage(self.voc).select(kind)
+        return SuperpositionVoltage(self.voc).select(kind)
     
 
-class sV(VoltageSource):
+class sV(VoltageSourceBase):
     """Arbitrary s-domain voltage source"""
 
     netkeyword = 's'
@@ -973,20 +966,20 @@ class sV(VoltageSource):
     def __init__(self, Vval):
 
         self.args = (Vval, )
-        Vval = sExpr(Vval)
-        self._Voc = Voltage(Vs(Vval))
+        Vval = LaplaceDomainExpression(Vval)
+        self._Voc = SuperpositionVoltage(LaplaceDomainExpression(Vval))
 
 
-class V(VoltageSource):
+class V(VoltageSourceBase):
     """Arbitrary voltage source"""
 
     def __init__(self, Vval):
 
         self.args = (Vval, )
-        self._Voc = Voltage(Vval)
+        self._Voc = SuperpositionVoltage(Vval)
 
         
-class Vstep(VoltageSource):
+class Vstep(VoltageSourceBase):
     """Step voltage source (s domain voltage of v / s)."""
 
     netkeyword = 'step'
@@ -994,12 +987,12 @@ class Vstep(VoltageSource):
     def __init__(self, v):
 
         self.args = (v, )
-        v = cExpr(v)
-        self._Voc = Voltage(tExpr(v) * Heaviside(t))
+        v = cexpr(v)
+        self._Voc = SuperpositionVoltage(TimeDomainExpression(v) * Heaviside(t))
         self.v0 = v
 
 
-class Vdc(VoltageSource):
+class Vdc(VoltageSourceBase):
     """DC voltage source (note a DC voltage source of voltage V has
     an s domain voltage of V / s)."""
 
@@ -1008,8 +1001,8 @@ class Vdc(VoltageSource):
     def __init__(self, v):
 
         self.args = (v, )
-        v = cExpr(v)
-        self._Voc = Voltage(Vconst(v, dc=True))
+        v = cexpr(v)
+        self._Voc = SuperpositionVoltage(cexpr(v, dc=True))
         self.v0 = v
 
     @property
@@ -1017,7 +1010,7 @@ class Vdc(VoltageSource):
         return self.v0
 
 
-class Vac(VoltageSource):
+class Vac(VoltageSourceBase):
     """AC voltage source."""
 
     netkeyword = 'ac'
@@ -1038,25 +1031,25 @@ class Vac(VoltageSource):
             
         if omega is None:
             omega = omega0sym
-        omega = Expr(omega)
+        omega = expr(omega)
 
-        V = Expr(V)
-        phi = Expr(phi)
+        V = cexpr(V)
+        phi = cexpr(phi)
 
         # Note, cos(-pi / 2) is not quite zero.
 
         self.omega = omega
         self.v0 = V
         self.phi = phi
-        self._Voc = Voltage(Vphasor(self.v0 * exp(j * self.phi),
-                                   ac=True, omega=self.omega))
+        self._Voc = SuperpositionVoltage(PhasorDomainVoltage(self.v0 * exp(j * self.phi),
+                                                             ac=True, omega=self.omega))
 
     @property
     def voc(self):
         return self.v0 * cos(self.omega * t + self.phi)
 
 
-class Vnoise(VoltageSource):
+class Vnoise(VoltageSourceBase):
     """Noise voltage source."""
 
     netkeyword = 'noise'
@@ -1064,22 +1057,22 @@ class Vnoise(VoltageSource):
 
     def __init__(self, V, nid=None):
 
-        V1 = Vnoisy(V, nid=nid)
+        V1 = AngularFourierNoiseDomainVoltage(V, nid=nid)
         self.args = (V, V1.nid)
-        self._Voc = Voltage(V1)
+        self._Voc = SuperpositionVoltage(V1)
 
         
-class v(VoltageSource):
+class v(VoltageSourceBase):
     """Arbitrary t-domain voltage source"""
 
     def __init__(self, vval):
 
         self.args = (vval, )
-        Vval = tExpr(vval)
-        self._Voc = Voltage(Vval)
+        Vval = TimeDomainExpression(vval)
+        self._Voc = SuperpositionVoltage(Vval)
 
 
-class CurrentSource(OnePort):
+class CurrentSourceBase(OnePort):
 
     current_source = True
     netname = 'I'
@@ -1093,10 +1086,10 @@ class CurrentSource(OnePort):
 
     def i_equation(self, v, kind='t'):
 
-        return Current(self.isc).select(kind)        
+        return SuperpositionCurrent(self.isc).select(kind)        
     
     
-class sI(CurrentSource):
+class sI(CurrentSourceBase):
     """Arbitrary s-domain current source"""
 
     netkeyword = 's'
@@ -1104,20 +1097,20 @@ class sI(CurrentSource):
     def __init__(self, Ival):
 
         self.args = (Ival, )
-        Ival = sExpr(Ival)
-        self._Isc = Current(Is(Ival))
+        Ival = LaplaceDomainExpression(Ival)
+        self._Isc = SuperpositionCurrent(LaplaceDomainExpression(Ival))
 
 
-class I(CurrentSource):
+class I(CurrentSourceBase):
     """Arbitrary current source"""
 
     def __init__(self, Ival):
 
         self.args = (Ival, )
-        self._Isc = Current(Ival)
+        self._Isc = SuperpositionCurrent(Ival)
 
             
-class Istep(CurrentSource):
+class Istep(CurrentSourceBase):
     """Step current source (s domain current of i / s)."""
 
     netkeyword = 'step'
@@ -1125,12 +1118,12 @@ class Istep(CurrentSource):
     def __init__(self, i):
 
         self.args = (i, )
-        i = cExpr(i)
-        self._Isc = Current(tExpr(i) * Heaviside(t))
+        i = cexpr(i)
+        self._Isc = SuperpositionCurrent(TimeDomainExpression(i) * Heaviside(t))
         self.i0 = i
 
 
-class Idc(CurrentSource):
+class Idc(CurrentSourceBase):
     """DC current source (note a DC current source of current i has
     an s domain current of i / s)."""
 
@@ -1139,8 +1132,8 @@ class Idc(CurrentSource):
     def __init__(self, i):
 
         self.args = (i, )
-        i = cExpr(i)
-        self._Isc = Current(Iconst(i, dc=True))
+        i = cexpr(i)
+        self._Isc = SuperpositionCurrent(cexpr(i, dc=True))
         self.i0 = i
 
     @property
@@ -1148,7 +1141,7 @@ class Idc(CurrentSource):
         return self.i0
 
 
-class Iac(CurrentSource):
+class Iac(CurrentSourceBase):
     """AC current source."""
 
     netkeyword = 'ac'
@@ -1169,23 +1162,23 @@ class Iac(CurrentSource):
             
         if omega is None:
             omega = omega0sym
-        omega = Expr(omega)
+        omega = cexpr(omega)
 
-        I = Expr(I)
-        phi = Expr(phi)
+        I = cexpr(I)
+        phi = cexpr(phi)
 
         self.omega = omega
         self.i0 = I
         self.phi = phi
-        self._Isc = Current(Iphasor(self.i0 * exp(j * self.phi),
-                                   ac=True, omega=self.omega))
+        self._Isc = SuperpositionCurrent(PhasorDomainCurrent(self.i0 * exp(j * self.phi),
+                                                       ac=True, omega=self.omega))
 
     @property
     def isc(self):
         return self.i0 * cos(self.omega * t + self.phi)
 
 
-class Inoise(CurrentSource):
+class Inoise(CurrentSourceBase):
     """Noise current source."""
 
     netkeyword = 'noise'
@@ -1193,19 +1186,19 @@ class Inoise(CurrentSource):
 
     def __init__(self, I, nid=None):
 
-        I1 = Inoisy(I, nid=nid)
-        self._Isc = Current(I1)
+        I1 = AngularFourierNoiseDomainCurrent(I, nid=nid)
+        self._Isc = SuperpositionCurrent(I1)
         self.args = (I, I1.nid)
 
         
-class i(CurrentSource):
+class i(CurrentSourceBase):
     """Arbitrary t-domain current source"""
 
     def __init__(self, ival):
 
         self.args = (ival, )
-        Ival = tExpr(ival)
-        self._Isc = Current(Ival)
+        Ival = TimeDomainExpression(ival)
+        self._Isc = SuperpositionCurrent(Ival)
 
 
 class Xtal(OnePort):
@@ -1218,10 +1211,10 @@ class Xtal(OnePort):
 
     def __init__(self, C0, R1, L1, C1):
 
-        self.C0 = cExpr(C0)
-        self.R1 = cExpr(R1)
-        self.L1 = cExpr(L1)
-        self.C1 = cExpr(C1)
+        self.C0 = cexpr(C0)
+        self.R1 = cexpr(R1)
+        self.L1 = cexpr(L1)
+        self.C1 = cexpr(C1)
 
         self._Z = self.expand().impedance
         self.args = (C0, R1, L1, C1)
@@ -1246,10 +1239,10 @@ class FerriteBead(OnePort):
 
     def __init__(self, Rs, Rp, Cp, Lp):
 
-        self.Rs = cExpr(Rs)
-        self.Rp = cExpr(Rp)
-        self.Cp = cExpr(Cp)
-        self.Lp = cExpr(Lp)
+        self.Rs = cexpr(Rs)
+        self.Rp = cexpr(Rp)
+        self.Cp = cexpr(Cp)
+        self.Lp = cexpr(Lp)
 
         self._Z = self.expand().impedance
         self.args = (Rs, Rp, Cp, Lp)
@@ -1313,8 +1306,8 @@ class CCVS(ControlledSource):
     def __init__(self, control, value):
 
         self.args = (control, value)        
-        self._Voc = Voltage(0)
-        self._Z = Impedance(0)
+        self._Voc = SuperpositionVoltage(0)
+        self._Z = impedance(0)
 
         
 class CCCS(ControlledSource):
@@ -1322,8 +1315,8 @@ class CCCS(ControlledSource):
     def __init__(self, control, value):
 
         self.args = (control, value)        
-        self._Isc = Current(0)
-        self._Y = Admittance(0)     
+        self._Isc = SuperpositionCurrent(0)
+        self._Y = admittance(0)     
 
         
 class VCVS(ControlledSource):
@@ -1331,8 +1324,8 @@ class VCVS(ControlledSource):
     def __init__(self, value):
 
         self.args = (value, )        
-        self._Voc = Voltage(0)
-        self._Z = Impedance(0)
+        self._Voc = SuperpositionVoltage(0)
+        self._Z = impedance(0)
 
         
 class VCCS(ControlledSource):
@@ -1340,8 +1333,8 @@ class VCCS(ControlledSource):
     def __init__(self, value):
 
         self.args = (value, )
-        self._Isc = Current(0)
-        self._Y = Admittance(0)     
+        self._Isc = SuperpositionCurrent(0)
+        self._Y = admittance(0)     
         
 
 class Dummy(OnePort):
@@ -1357,7 +1350,7 @@ class K(Dummy):
     def __init__(self, L1, L2, K):
 
         self.args = (L1, L2, K)
-        self.K = cExpr(K)
+        self.K = cexpr(K)
 
 
 class W(Dummy):
@@ -1366,7 +1359,7 @@ class W(Dummy):
     def __init__(self):
 
         self.args = ()
-        self._Z = Impedance(0)
+        self._Z = impedance(0)
 
         
 class O(Dummy):
@@ -1375,7 +1368,7 @@ class O(Dummy):
     def __init__(self):
 
         self.args = ()
-        self._Y = Admittance(0)
+        self._Y = admittance(0)
 
         
 class P(O):
@@ -1453,12 +1446,12 @@ def ladder(*args, **kwargs):
     
 # Imports at end to circumvent circular dependencies
 from .expr import Expr, expr
-from .cexpr import cExpr, Iconst, Vconst
-from .sexpr import sExpr, Is, Vs, Ys, Zs
-from .texpr import tExpr
-from .noiseomegaexpr import Inoisy, Vnoisy
-from .voltage import Voltage
-from .current import Current
-from .phasor import Iphasor, Vphasor
+from .cexpr import cexpr
+from .sexpr import LaplaceDomainExpression
+from .texpr import TimeDomainExpression
+from .noiseomegaexpr import AngularFourierNoiseDomainCurrent, AngularFourierNoiseDomainVoltage
+from .superpositionvoltage import SuperpositionVoltage
+from .superpositioncurrent import SuperpositionCurrent
+from .phasor import PhasorDomainCurrent, PhasorDomainVoltage
 from .twoport import Ladder, LSection, TSection
 
