@@ -1,14 +1,14 @@
-"""This module provides the sExpr class to represent s-domain (Laplace
-domain) expressions.
+"""This module provides the LaplaceDomainExpression class to represent
+s-domain (Laplace domain) expressions.
 
-Copyright 2014--2020 Michael Hayes, UCECE
+Copyright 2014--2021 Michael Hayes, UCECE
 
 """
 
 from __future__ import division
+from .domains import LaplaceDomain
 from .laplace import inverse_laplace_transform
 from .sym import ssym, tsym, j, pi, sympify
-from .vector import Vector
 from .ratfun import _zp2tf, _pr2tf, Ratfun
 from .expr import Expr, symbol, expr, ExprDict, exprcontainer
 from .functions import sqrt
@@ -16,10 +16,10 @@ import numpy as np
 from sympy import limit, exp, Poly, Integral, div, oo, Eq, Expr as symExpr
 
 
-__all__ = ('Hs', 'Is', 'Vs', 'Ys', 'Zs', 'zp2tf', 'tf', 'pr2tf')
+__all__ = ('sexpr', 'zp2tf', 'tf', 'pr2tf')
 
 
-class sExpr(Expr):
+class LaplaceDomainExpression(LaplaceDomain, Expr):
     """s-domain expression or symbol."""
 
     var = ssym
@@ -27,8 +27,7 @@ class sExpr(Expr):
     def __init__(self, val, **assumptions):
 
         check = assumptions.pop('check', True)                
-        super(sExpr, self).__init__(val, **assumptions)
-        self._laplace_conjugate_class = tExpr
+        super(LaplaceDomainExpression, self).__init__(val, **assumptions)
 
         expr = self.expr        
         if check and expr.find(tsym) != set() and not expr.has(Integral):
@@ -38,6 +37,9 @@ class sExpr(Expr):
             raise ValueError(
                 's-domain expression %s cannot depend on t' % self.expr)
 
+    def as_expr(self):
+        return LaplaceDomainExpression(self)
+
     @classmethod
     def from_poles_residues(cls, poles, residues):
         """Create a transfer function from lists of poles and residues.
@@ -107,23 +109,33 @@ class sExpr(Expr):
 
         """
 
-        assumptions = self.merge_assumptions(**assumptions)
+        assumptions = self.assumptions.merge(**assumptions)
         result = inverse_laplace_transform(self.expr, self.var, tsym,
                                            **assumptions)
 
-        if hasattr(self, '_laplace_conjugate_class'):
-            result = self._laplace_conjugate_class(result)
-        else:
-            result = tExpr(result)
-        return result
+        return self.change(result, domain='time', **assumptions)
 
     def ILT(self, **assumptions):
-        """Convert to t-domain.   This is an alias for inverse_laplace."""
+        """Attempt inverse Laplace transform.
+
+        If causal=True the response is zero for t < 0 and
+        the result is multiplied by Heaviside(t)
+        If ac=True or dc=True the result is extrapolated for t < 0.
+        Otherwise the result is only known for t >= 0.
+
+        """
 
         return self.inverse_laplace(**assumptions)
-    
+        
     def time(self, **assumptions):
-        """Convert to time domain."""
+        """Convert to time domain.
+
+        If causal=True the response is zero for t < 0 and
+        the result is multiplied by Heaviside(t)
+        If ac=True or dc=True the result is extrapolated for t < 0.
+        Otherwise the result is only known for t >= 0.
+
+        """
 
         try:
             return self.inverse_laplace(**assumptions)
@@ -133,21 +145,27 @@ class sExpr(Expr):
     def laplace(self, **assumptions):
         """Convert to s-domain."""
 
-        assumptions = self.merge_assumptions(**assumptions)
+        assumptions = self.assumptions.merge(**assumptions)
         return self.__class__(self, **assumptions)
     
     def fourier(self, **assumptions):
         """Convert to Fourier domain."""
         from .symbols import f
         
-        if assumptions.get('causal', self.is_causal):
-            return self.subs(j * 2 * pi * f)
+        result = self.time(**assumptions).fourier(**assumptions)
+        return result
 
-        return self.time(**assumptions).fourier(**assumptions)
+    def angular_fourier(self, **assumptions):
+        """Convert to angular Fourier domain."""
+        
+        result = self.time(**assumptions).angular_fourier(**assumptions)
+        return result
 
     def phasor(self, **assumptions):
+        """Convert to phasor domain."""
 
-        return self.time(**assumptions).phasor(**assumptions)
+        result = PhasorDomainFrequencyExpression.from_laplace(self, **assumptions)
+        return self.change(result, domain='phasor', **assumptions)
 
     def transient_response(self, tvector=None):
         """Evaluate transient (impulse) response."""
@@ -215,7 +233,7 @@ class sExpr(Expr):
 
         # Evaluate transient response.
         th = np.arange(N) * dt - dt
-        h = sExpr(expr).transient_response(th)
+        h = LaplaceDomainExpression(expr).transient_response(th)
 
         print('Convolving...')
         ty = t
@@ -272,11 +290,11 @@ class sExpr(Expr):
         lhs = (N * Y).ILT(causal=True)
         rhs = (D * X).ILT(causal=True)
 
-        return tExpr(Eq(lhs.expr, rhs.expr))
+        return TimeDomainExpression(Eq(lhs.expr, rhs.expr))
 
     def evaluate(self, svector=None):
 
-        return super(sExpr, self).evaluate(svector)
+        return super(LaplaceDomainExpression, self).evaluate(svector)
 
     def plot(self, t=None, **kwargs):
         """Plot pole-zero map."""
@@ -316,10 +334,10 @@ class sExpr(Expr):
         """
 
         def def1(defs, symbolname, value):
-            from .cexpr import cExpr
+            from .cexpr import ConstantExpression
             
             sym1 = symbol(symbolname)
-            defs[symbolname] = cExpr(value)
+            defs[symbolname] = ConstantExpression(value)
             return sym1
 
         factors = self.as_ordered_factors()
@@ -408,232 +426,6 @@ class sExpr(Expr):
         from .discretetime import z, dt
         
         return self.subs((1 / dt) * (1 - z**-1))        
-    
-    def transform(self, arg, **assumptions):
-        """Transform into a different domain."""
-
-        from .fexpr import fExpr, f
-        from .omegaexpr import omegaExpr, omega
-        from .symbols import jomega        
-
-        arg = expr(arg)
-
-        is_causal = assumptions.get('causal', self.is_causal)
-        
-        if arg.has(jomega):
-            if not is_causal:
-                raise ValueError('Cannot convert non-causal s-expression to jomega domain')
-            result = self.subs(jomega)
-            # Handle args like 5 * jomega,  This might be too cute.
-            return result.subs(arg / j, **assumptions)
-
-        elif isinstance(arg, omegaExpr):
-            if not is_causal:
-                raise ValueError('Cannot convert non-causal s-expression to omega domain')
-            result = self.subs(jomega)
-            # Handle args like 5 * omega,  This might be too cute.
-            return result.subs(arg, **assumptions)
-
-        elif isinstance(arg, fExpr):
-            if not is_causal:
-                raise ValueError('Cannot convert non-causal s-expression to f domain')
-            result = self.subs(j * 2 * pi * f)
-            # Handle args like 5 * f,  This might be too cute.
-            return result.subs(arg, **assumptions)        
-        
-        return super(sExpr, self).transform(arg, **assumptions)    
-
-    
-# Perhaps use a factory to create the following classes?
-
-class Zs(sExpr):
-
-    """s-domain impedance value."""
-
-    quantity = 'Impedance'
-    units = 'ohms'
-
-    def __init__(self, val, causal=True, **assumptions):
-
-        super(Zs, self).__init__(val, causal=causal, **assumptions)
-        self._laplace_conjugate_class = Zt
-
-    def cpt(self):
-        from .oneport import R, C, L, Z
-
-        if self.is_number or self.is_dc:
-            return R(self.expr)
-
-        z = self * s
-
-        if z.is_number:
-            return C((1 / z).expr)
-
-        z = self / s
-
-        if z.is_number:
-            return L(z.expr)
-
-        return Z(self)
-
-    def network(self, form='default'):
-        """Synthesise a network with an equivalent impedance.
-        `form` includes: cauerI, cauerII, fosterI, fosterII.
-
-        Note some methods generate networks with negative value
-        components."""
-
-        from .synthesis import network
-        
-        return network(self, form)
-
-
-class Ys(sExpr):
-
-    """s-domain admittance value."""
-
-    quantity = 'Admittance'
-    units = 'siemens'
-
-    def __init__(self, val, causal=True, **assumptions):
-
-        super(Ys, self).__init__(val, causal=causal, **assumptions)
-        self._laplace_conjugate_class = Yt
-
-    def cpt(self):
-        from .oneport import G, C, L, Y
-
-        if self.is_number or self.is_dc:
-            return G(self.expr)
-
-        y = self * s
-
-        if y.is_number:
-            return L((1 / y).expr)
-
-        y = self / s
-
-        if y.is_number:
-            return C(y.expr)
-
-        return Y(self)
-
-    def network(self, form='default'):
-        """Synthesise a network with an equivalent impedance.
-        `form` includes: cauerI, cauerII, fosterI, fosterII.
-
-        Note some methods generate networks with negative value
-        components."""        
-
-        from .synthesis import network
-        
-        return network(1 / self, form)
-
-    
-class Vs(sExpr):
-
-    """s-domain voltage (units V s / radian)."""
-
-    quantity = 's-Voltage'
-    units = 'V/Hz'
-    superkind = 'Voltage'
-
-    def __init__(self, val, **assumptions):
-
-        super(Vs, self).__init__(val, **assumptions)
-        self._laplace_conjugate_class = Vt
-
-    def cpt(self):
-        from .oneport import V
-        return V(self)
-
-    def __mul__(self, x):
-        """Multiply"""
-
-        if isinstance(x, Ys):
-            return Is(super(Vs, self).__mul__(x))
-        if isinstance(x, (cExpr, sExpr, symExpr, int, float, complex)):
-            return super(Vs, self).__mul__(x)
-        self._incompatible(x, '*')
-
-    def __truediv__(self, x):
-        """Divide"""
-
-        if isinstance(x, Zs):
-            return Is(super(Vs, self).__truediv__(x))
-        if isinstance(x, (cExpr, sExpr, symExpr, int, float, complex)):
-            return super(Vs, self).__truediv__(x)
-        self._incompatible(x, '/')        
-
-        
-class Is(sExpr):
-
-    """s-domain current (units A s / radian)."""
-
-    quantity = 's-Current'
-    units = 'A/Hz'
-    superkind = 'Current'
-
-    def __init__(self, val, **assumptions):
-
-        super(Is, self).__init__(val, **assumptions)
-        self._laplace_conjugate_class = It
-
-    def cpt(self):
-        from .oneport import I
-        
-        return I(self)
-
-    def __mul__(self, x):
-        """Multiply"""
-
-        if isinstance(x, Zs):
-            return Vs(super(Is, self).__mul__(x))            
-        if isinstance(x, (cExpr, sExpr, symExpr, int, float, complex)):
-            return super(Is, self).__mul__(x)
-        self._incompatible(x, '*')        
-
-    def __truediv__(self, x):
-        """Divide"""
-
-        if isinstance(x, Ys):
-            return Vs(super(Is, self).__truediv__(x))
-        if isinstance(x, (cExpr, sExpr, symExpr, int, float, complex)):
-            return super(Is, self).__truediv__(x)
-        self._incompatible(x, '/')                
-    
-
-class Hs(sExpr):
-
-    """s-domain ratio"""
-
-    quantity = 's-ratio'
-    units = ''
-
-    def __init__(self, val, **assumptions):
-
-        super(Hs, self).__init__(val, **assumptions)
-        self._laplace_conjugate_class = Ht
-
-        
-class VsVector(Vector):
-
-    _typewrap = Vs
-
-
-class IsVector(Vector):
-
-    _typewrap = Is
-
-
-class YsVector(Vector):
-
-    _typewrap = Ys
-
-
-class ZsVector(Vector):
-
-    _typewrap = Zs
 
 
 def tf(numer, denom=1, var=None):
@@ -646,7 +438,7 @@ def tf(numer, denom=1, var=None):
     N = Poly(sympify(numer), var)
     D = Poly(sympify(denom), var)
 
-    return Hs(N / D, causal=True)
+    return LaplaceDomainTransferFunction(N / D, causal=True)
 
 
 def zp2tf(zeros, poles, K=1, var=None):
@@ -655,7 +447,7 @@ def zp2tf(zeros, poles, K=1, var=None):
 
     if var is None:
         var = ssym
-    return Hs(_zp2tf(sympify(zeros), sympify(poles),
+    return LaplaceDomainTransferFunction(_zp2tf(sympify(zeros), sympify(poles),
                      sympify(K), var), causal=True)
 
 
@@ -664,18 +456,32 @@ def pr2tf(poles, residues, var=None):
 
     if var is None:
         var = ssym
-    return Hs(_pr2tf(sympify(poles), sympify(residues), var), causal=True)
+    return LaplaceDomainTransferFunction(_pr2tf(sympify(poles), sympify(residues), var), causal=True)
 
 
 def sexpr(arg, **assumptions):
-    """Create sExpr object.  If `arg` is ssym return s"""
+    """Create LaplaceDomainExpression object.  If `arg` is ssym return s"""
 
     if arg is ssym:
         return s
-    return sExpr(arg, **assumptions)
+    return LaplaceDomainExpression(arg, **assumptions)
 
 
-from .texpr import Ht, It, Vt, Yt, Zt, tExpr, texpr
-from .cexpr import cExpr
-s = sExpr('s')
+from .expressionclasses import expressionclasses
+classes = expressionclasses.make(LaplaceDomainExpression)
+
+LaplaceDomainVoltage = classes['voltage']
+LaplaceDomainCurrent = classes['current']
+LaplaceDomainAdmittance = classes['admittance']
+LaplaceDomainImpedance = classes['impedance']
+LaplaceDomainTransferFunction = classes['transfer']
+expressionclasses.add('laplace', classes)
+
+
+from .texpr import TimeDomainImpulseResponse, TimeDomainAdmittance, TimeDomainImpedance
+from .texpr import TimeDomainExpression, TimeDomainVoltage, TimeDomainCurrent, texpr
+from .cexpr import ConstantExpression
+from .phasor import PhasorDomainFrequencyExpression
+
+s = LaplaceDomainExpression('s')
 
