@@ -7,8 +7,8 @@ Copyright 2019-2020 Michael Hayes, UCECE
 
 from .mnacpts import I, V, C, L
 from .matrix import Matrix
-from .smatrix import sMatrix
-from .tmatrix import tMatrix
+from .smatrix import LaplaceDomainMatrix
+from .tmatrix import TimeDomainMatrix
 from .sym import sympify, ssym
 import sympy as sym
 
@@ -160,7 +160,7 @@ class StateSpace(object):
                     continue
                 yexprs.append(self.sscct[node].v.subs(subsdict).expand().expr)
                 # Note, this can introduce a name conflict
-                y.append(Vt('v_%s(t)' % node))
+                y.append(TimeDomainVoltage('v_%s(t)' % node))
 
         if branch_currents:
             for name in cct.branch_list:
@@ -168,7 +168,7 @@ class StateSpace(object):
                 # state variable?
                 name2 = cpt_map[name]                    
                 yexprs.append(self.sscct[name2].i.subs(subsdict).expand().expr)
-                y.append(It('i_%s(t)' % name))                    
+                y.append(TimeDomainCurrent('i_%s(t)' % name))                    
 
         Cmat, b = sym.linear_eq_to_matrix(yexprs, *statesyms)
         if sourcesyms != []:        
@@ -181,13 +181,13 @@ class StateSpace(object):
         _hack_vars(sources)
         
         # Note, Matrix strips the class from each element...
-        self.x = tMatrix(statevars)
+        self.x = TimeDomainMatrix(statevars)
 
         self.x0 = Matrix(initialvalues)
         
-        self.dotx = tMatrix([sym.Derivative(x1, t) for x1 in self.x])
+        self.dotx = TimeDomainMatrix([sym.Derivative(x1, t) for x1 in self.x])
 
-        self.u = tMatrix(sources)
+        self.u = TimeDomainMatrix(sources)
 
         self.A = Matrix(A)
         self.B = Matrix(B)        
@@ -206,9 +206,9 @@ class StateSpace(object):
         where x is the state vector and u is the input vector.
         """
         
-        return tExpr(sym.Eq(self.dotx, sym.MatAdd(sym.MatMul(self.A, self.x),
-                                                  sym.MatMul(self.B, self.u)),
-                            evaluate=False))
+        return TimeDomainExpression(sym.Eq(self.dotx, sym.MatAdd(sym.MatMul(self.A, self.x),
+                                                                 sym.MatMul(self.B, self.u)),
+                                           evaluate=False))
 
     def output_equations(self):
         """System of output equations:
@@ -220,57 +220,57 @@ class StateSpace(object):
 
         """
         
-        return tExpr(sym.Eq(self.y, sym.MatAdd(sym.MatMul(self.C, self.x),
-                                               sym.MatMul(self.D, self.u)),
-                            evaluate=False))
+        return TimeDomainExpression(sym.Eq(self.y, sym.MatAdd(sym.MatMul(self.C, self.x),
+                                                              sym.MatMul(self.D, self.u)),
+                                           evaluate=False))
 
     @property
     def Phi(self):
         """s-domain state transition matrix."""
 
-        M = sMatrix(sym.eye(len(self.x)) * ssym - self.A)
-        return sMatrix(M.inv().canonical())
+        M = LaplaceDomainMatrix(sym.eye(len(self.x)) * ssym - self.A)
+        return LaplaceDomainMatrix(M.inv().canonical())
 
     @property
     def phi(self):
         """State transition matrix."""        
-        return tMatrix(self.Phi.inverse_laplace(causal=True))
+        return TimeDomainMatrix(self.Phi.inverse_laplace(causal=True))
         
     @property
     def U(self):
         """Laplace transform of input vector."""
-        return sMatrix(self.u.laplace())
+        return LaplaceDomainMatrix(self.u.laplace())
 
     @property
     def X(self):
         """Laplace transform of state-variable vector."""        
-        return sMatrix(self.x.laplace())
+        return LaplaceDomainMatrix(self.x.laplace())
 
     @property
     def Y(self):
         """Laplace transform of output vector."""        
-        return sMatrix(self.y.laplace())
+        return LaplaceDomainMatrix(self.y.laplace())
 
     @property
     def H(self):
         """X(s) / U(s)"""
 
-        return sMatrix(self.Phi * self.B).canonical()
+        return LaplaceDomainMatrix(self.Phi * self.B).canonical()
 
     @property
     def h(self):
-        return tMatrix(self.H.inverse_laplace(causal=True))
+        return TimeDomainMatrix(self.H.inverse_laplace(causal=True))
 
     @property
     def G(self):
         """System transfer functions."""
 
-        return sMatrix(self.C * self.H + self.D).canonical()
+        return LaplaceDomainMatrix(self.C * self.H + self.D).canonical()
 
     @property
     def g(self):
         """System impulse responses."""        
-        return tMatrix(self.G.inverse_laplace(causal=True))
+        return TimeDomainMatrix(self.G.inverse_laplace(causal=True))
     
     def characteristic_polynomial(self):
         """Characteristic polynomial (aka system polynomial).
@@ -278,7 +278,7 @@ class StateSpace(object):
         `lambda(s) = |s * I - A|`"""
 
         M = Matrix(sym.eye(len(self.x)) * ssym - self.A)        
-        return sExpr(M.det()).simplify()
+        return LaplaceDomainExpression(M.det()).simplify()
 
     @property
     def P(self):
@@ -321,7 +321,7 @@ class StateSpace(object):
         # return L
         
         e = self.eigenvalues
-        return sMatrix(sym.diag(*e))
+        return LaplaceDomainMatrix(sym.diag(*e))
 
     @property        
     def eigenvectors(self):
@@ -336,10 +336,10 @@ class StateSpace(object):
 
         E, L = self.A.diagonalize()
         
-        return sMatrix(E)
+        return LaplaceDomainMatrix(E)
     
     
 from .symbols import t, s
 from .expr import ExprList
-from .texpr import Vt, It, tExpr
-from .sexpr import sExpr
+from .texpr import TimeDomainVoltage, TimeDomainCurrent, TimeDomainExpression
+from .sexpr import LaplaceDomainExpression
