@@ -2,19 +2,22 @@
 Current and Voltage.  It represents voltages and currents as a
 superposition in different transform domains.
 
-Copyright 2019--2020 Michael Hayes, UCECE
+Copyright 2019--2021 Michael Hayes, UCECE
 
 """
 
 from __future__ import division
+import sympy as sym
 from .expr import Expr, ExprDict, expr
 from .sym import tsym, omega0sym, symbols_find, is_sympy, symsymbol
 from .acdc import is_ac
+from .domains import SuperpositionDomain
 from .printing import pprint, pretty, latex
+from .classmap import domain_kind_quantity_to_class, expr_to_domain_kind
+from .classmap import domain_quantity_to_class
 
-__all__ = ('Super', 'Voltage', 'Current')
 
-class Super(ExprDict):
+class Superposition(SuperpositionDomain, ExprDict):
     """This class represents a superposition of different signal types,
     DC, AC, transient, and noise.
     
@@ -53,9 +56,13 @@ class Super(ExprDict):
     # It can decompose a signal into AC, DC, and transient components.
     # The 't' key is the transient component viewed in the time domain.
     # The 's' key is the transient component viewed in the Laplace domain.    
-    
+
+    is_voltage = False
+    is_current = False
+
+
     def __init__(self, *args, **kwargs):
-        super (Super, self).__init__()
+        super (Superposition, self).__init__()
 
         for arg in args:
             self.add(arg)
@@ -64,15 +71,19 @@ class Super(ExprDict):
         if not any(self):
             return 0
 
-        decomp = self.decompose()
+        if len(self) == 1:
+            decomp = self
+        else:
+            decomp = self.decompose()
         
         # If have a single element, show value.
         if len(decomp) == 1:
             key = list(decomp)[0]
             if key in ('dc', 't'):
                 return decomp[key]
-            if key == 's':
-                if decomp[key].has(s):
+            if key in ('s', 'f', 'omega'):
+                expr = decomp[key]
+                if expr.has(expr.var):
                     return decomp[key]
                 # Have something like s * 0 + 1 so don't display as 1
                 # since not a dc value.
@@ -82,7 +93,7 @@ class Super(ExprDict):
         # using a decomposition in the transform domains.
         # We could present the result in the time-domain but this
         # hides the underlying way the signal is analysed.
-        return decomp
+        return dict(decomp)
 
     def latex(self, **kwargs):
         """Latex"""
@@ -96,7 +107,7 @@ class Super(ExprDict):
 
     def _repr_latex_(self):
         """This is used by jupyter notebooks to display an expression using
-        LaTeX markup.  However, this requires matjax.  If this method
+        LaTeX markup.  However, this requires mathjax.  If this method
         is not defined, jupyter falls back on _repr__pretty_ which
         outputs unicode."""
 
@@ -113,7 +124,7 @@ class Super(ExprDict):
         if isinstance(key, Expr):
             key = key.expr
         if key in self:
-            return super(Super, self).__getitem__(key)
+            return super(Superposition, self).__getitem__(key)
         decomp = self.decompose()
         if key == 0 and 'dc' in decomp:
             key = 'dc'
@@ -122,6 +133,11 @@ class Super(ExprDict):
         
         return decomp.__getitem__(key)        
 
+    def decompose_to_domain(self, expr, kind):
+
+        cls = domain_kind_quantity_to_class(kind, self.quantity)
+        return cls(expr)
+    
     @property
     def symbols(self):
         """Return dictionary of symbols in the expression keyed by name."""
@@ -209,7 +225,7 @@ class Super(ExprDict):
         return self.has_ac and self.ac_keys() == list(self.keys())
 
     @property
-    def is_phasor(self):
+    def is_phasor_domain(self):
         """True if has single AC component."""
 
         return self.ac and len(list(self.keys())) == 1
@@ -301,14 +317,22 @@ class Super(ExprDict):
     def __add__(self, x):
 
         def _is_s_arg(x):
-            return isinstance(x, sExpr) or (isinstance(x, Super) and 's' in x)
+            return isinstance(x, LaplaceDomainExpression) or (isinstance(x, Superposition) and 's' in x)
+
+        x = expr(x)
+        # Perhaps check if a constant?
+        if x.is_undefined:
+            x = x.as_quantity(self.quantity)
+        
+        if x.quantity != self.quantity:
+            raise ValueError('Incompatible quantities %s and %s' % (self.quantity, x.quantity))
 
         if _is_s_arg(x):
             new = self.decompose()
         else:
             new = self.__class__(self)            
 
-        if isinstance(x, Super):
+        if isinstance(x, Superposition):
             for value in x.values():
                 new.add(value)
         else:
@@ -362,36 +386,40 @@ class Super(ExprDict):
                 return True
         return False    
 
-    def _decompose(self, expr):
-        """Decompose a t-domain expr into AC, DC, and s-domain
+    def _decompose_timedomain_expr(self, expr):
+        """Decompose a time domain expr into AC, DC, and s-domain
         transient components."""
 
+        result = {}
+        
         # Extract DC components
         dc = expr.expr.coeff(tsym, 0)
         if dc != 0:
-            self['dc'] = cExpr(dc)
+            result['dc'] = ConstantExpression(dc)
             expr -= dc
 
         if expr == 0:
-            return
+            return result
 
         # Extract AC components        
         ac = 0
         terms = expr.expr.as_ordered_terms()
         for term in terms:
             if is_ac(term, tsym):
-                self.add(tExpr(term).phasor())
-                ac += term
+                 p = TimeDomainExpression(term).phasor()
+                 result[p.omega] = p
+                 ac += term
 
         expr -= ac
         if expr == 0:
-            return
+            return result
 
         # The remaining components are considered transient
         # so convert to Laplace representation.
         sval = expr.laplace()
 
-        self['s'] = sval
+        result['s'] = sval
+        return result
 
     def decompose(self):
         """Decompose into a new representation in the transform domains."""
@@ -400,11 +428,15 @@ class Super(ExprDict):
             return self._decomposition
 
         new = self.__class__()
-        if 't' in self:
-            new._decompose(self['t'])
+
         for kind, value in self.items():
-            if kind != 't':
+            if kind == 't':
+                decomp = new._decompose_timedomain_expr(value)
+                for value in decomp.values():
+                    new.add(value)
+            else:
                 new.add(value)
+                
         self._decomposition = new                
         return new
     
@@ -415,7 +447,6 @@ class Super(ExprDict):
         'laplace' :  the laplace domain representation (equivalent to self.laplace())
         'ivp' :  the s-domain representation (equivalent to self.laplace())
         'dc' : the DC component
-        'ac' : the AC component with angular frequency omega0
         omega : the AC component with angular frequency omega
         's' : the transient component in the s-domain
         'n' : the noise component
@@ -423,45 +454,47 @@ class Super(ExprDict):
               include the DC and AC components).
 
         """
-        if kind == 'super':
-            return self
-        elif kind == 'time':
-            return self.time()
-        elif kind in ('ivp', 'laplace'):
-            return self.laplace()
 
-        if isinstance(kind, str) and kind[0] == 'n':
-            if kind not in self:
-                return self.decompose_domains['n'](0)
-            return self[kind]
+        if isinstance(kind, str):
+            if kind == 'super':
+                return self
+            elif kind == 'time':
+                return self.time()
+            elif kind in ('ivp', 'laplace'):
+                return self.laplace()
+
+            if isinstance(kind, str) and kind[0] == 'n':
+                if kind not in self:
+                    return self.decompose_to_domain(0, 'n')
+                return self[kind]
         
         obj = self
         if 't' in self and (kind == omega or 't' != kind):
             # The rationale here is that there may be
             # DC and AC components included in the 't' part.
             obj = self.decompose()
-            
+
         if kind not in obj:
-            if kind not in obj.decompose_domains:
-                kind = 'ac'
-            return obj.decompose_domains[kind](0)
+            return obj.decompose_to_domain(0, kind)
         return obj[kind]
 
     def netval(self, kind):
 
         def kind_keyword(kind):
-            if isinstance(kind, str) and kind[0] == 'n':
+            if not isinstance(kind, str):
+                return 'ac'
+            
+            if kind[0] == 'n':
                 return 'noise'
             elif kind in ('ivp', 'laplace'):
                 return 's'
             elif kind in ('t', 'time'):
                 return ''                
-            elif not isinstance(kind, str):
-                return 'ac'
             return kind
 
         val = self.select(kind)
-        if kind in ('s', 'ivp') and (val.is_causal or val.is_dc or val.is_ac):
+        if (isinstance(kind, str) and kind in ('s', 'ivp') and
+            (val.is_causal or val.is_dc or val.is_ac)):
             # Convert to time representation so that can re-infer
             # causality, etc.
             return val.time()
@@ -477,7 +510,7 @@ class Super(ExprDict):
         return keyword, val
     
     def _kind(self, value):
-        if isinstance(value, Phasor):
+        if isinstance(value, PhasorDomainExpression):
             # Use angular frequency for key.  This can be a nuisance
             # for numerical values since cannot use x.3 syntax
             # say for an angular frequency of 3.
@@ -486,10 +519,8 @@ class Super(ExprDict):
                 key = key.expr
             return key
 
-        for kind, mtype in self.decompose_domains.items():
-            if isinstance(value, mtype):
-                return kind
-        return None
+        kind = expr_to_domain_kind(value)
+        return kind
 
     def _parse(self, string):
         """Parse t or s-domain expression or symbol, interpreted in time
@@ -500,19 +531,19 @@ class Super(ExprDict):
 
         symbols = symbols_find(string)
         if 's' in symbols:
-            return self.add(sExpr(string))
+            return self.add(LaplaceDomainExpression(string))
 
         if 'omega' in symbols:
             if 't' in symbols:
                 # Handle cos(omega * t)
-                return self.add(tExpr(string))
-            return self.add(omegaExpr(string))
+                return self.add(TimeDomainExpression(string))
+            return self.add(AngularFourierDomainExpression(string))
 
         if 'f' in symbols:
             # TODO, handle AC of different frequency
-            return self.add(fExpr(string))
+            return self.add(FourierDomainExpression(string))
 
-        return self.add(tExpr(string))
+        return self.add(TimeDomainExpression(string))
 
     def _add_noise(self, value):
 
@@ -526,19 +557,20 @@ class Super(ExprDict):
     def add(self, value):
         """Add a value into the superposition."""
 
+        try:
+            val = value.expr
+        except:
+            val = value
+        
         # Avoid triggering __eq__ for Super otherwise have infinite recursion
-        if not isinstance(value, Super):
-            try:
-                val = value.expr
-            except:
-                val = value
+        if not isinstance(value, Superposition):
             if val == 0:
                 return
 
-        if '_decomposition' in self:
+        if hasattr(self, '_decomposition'):            
             delattr(self, '_decomposition')
 
-        if isinstance(value, Super):
+        if isinstance(value, Superposition):
             for kind, value in value.items():
                 self.add(value)
             return
@@ -546,35 +578,21 @@ class Super(ExprDict):
         if isinstance(value, str):
             return self._parse(value)
 
-        if isinstance(value, noiseExpr):
+        if isinstance(value, NoiseExpression):
             self._add_noise(value)
             return
-        
-        # DC should be real but allow const complex value.
-        if isinstance(value, (int, float, complex)):
-            value = self.decompose_domains['dc'](value)
-        elif is_sympy(value):
-            try:
-                # Look for I, 5 * I, etc.
-                if value.is_constant:
-                    value = self.decompose_domains['dc'](value)
-            except:
-                pass
 
-        # TODO, perhaps handle Fourier domain expressions in the
-        # decomposition?  For now, convert to time domain.
-        if isinstance(value, (fExpr, omegaExpr)):
-            value = value.time()
+        value = expr(value)
+        if value.has(sym.Float):
+            # Avoid 8.00000000000, etc.
+            value.expr = sym.nsimplify(value.expr)
 
         kind = self._kind(value)
         if kind is None:
-            if value.__class__ in self.type_map:
-                value = self.type_map[value.__class__](value)
+            if self.is_voltage:
+                value = value.as_voltage()
             else:
-                for cls1, cls2 in self.type_map.items():
-                    if isinstance(value, cls1):
-                        value = cls2(value)
-                        break
+                value = value.as_current()                    
             kind = self._kind(value)
 
         if kind is None:
@@ -671,7 +689,7 @@ phasor, for example, using: %s""" % foo)
 
     @property
     def n(self):
-        result = self.decompose_domains['n'](0)
+        result = self.decompose_to_domain(0, 'n')
         for key in self.noise_keys():
             result += self[key]
         return result
@@ -681,23 +699,14 @@ phasor, for example, using: %s""" % foo)
         """Return the total noise."""
         return self.n
 
-    @property
-    def w(self):
-        """Return the AC component with angular frequency omega0.
-        This should be deprecated."""
-        return self.select(omega0sym)    
-
     def time(self, **assumptions):
         """Convert to time domain."""
 
-        result = self.time_class(0)
+        result = domain_quantity_to_class('time', quantity=self.quantity)(0)
 
         # TODO, integrate noise
         for val in self.values():
-            if hasattr(val, 'time'):
-                result += val.time(**assumptions)
-            else:
-                result += val
+            result += val.time(**assumptions)
         return result
 
     def transient_response(self, tvector=None):
@@ -726,7 +735,7 @@ phasor, for example, using: %s""" % foo)
     def laplace(self, **assumptions):
         """Convert to s-domain."""                
 
-        result = self.laplace_class(0)
+        result = domain_quantity_to_class('laplace', quantity=self.quantity)(0)
         for val in self.values():
             result += val.laplace(**assumptions)
         return result
@@ -734,8 +743,16 @@ phasor, for example, using: %s""" % foo)
     def fourier(self, **assumptions):
         """Convert to Fourier domain."""        
 
+        result = domain_quantity_to_class('fourier', quantity=self.quantity)(0)
+        for val in self.values():
+            result += val.fourier(**assumptions)
+        return result
+
+    def angular_fourier(self, **assumptions):
+        """Convert to angular Fourier domain."""        
+
         # TODO, could optimise.
-        return self.time(**assumptions).fourier(**assumptions)
+        return self.time(**assumptions).angular_fourier(**assumptions)    
 
     def canonical(self):
         new = self.__class__()
@@ -755,13 +772,33 @@ phasor, for example, using: %s""" % foo)
         """Create oneport component."""
         return self.cpt()
 
+    def rms(self):
+        return self.n.rms()
+
+    def asd(self):
+        return self.n.asd()
+
+    def psd(self):
+        return self.n.psd()        
+
     
-    
-from .cexpr import cExpr        
-from .fexpr import fExpr    
-from .sexpr import sExpr
-from .texpr import tExpr
-from .noiseexpr import noiseExpr
-from .phasor import Phasor
-from .omegaexpr import omegaExpr
+from .cexpr import ConstantExpression        
+from .fexpr import FourierDomainExpression    
+from .sexpr import LaplaceDomainExpression
+from .texpr import TimeDomainExpression
+from .noiseexpr import NoiseExpression
+from .phasor import PhasorDomainExpression
+from .omegaexpr import AngularFourierDomainExpression
 from .symbols import s, omega
+
+
+# See expr.py
+
+import sys
+try:
+    from .printing import latex
+    formatter = sys.displayhook.shell.display_formatter.formatters['text/latex']
+    
+    formatter.type_printers[Superposition] = Superposition._repr_latex_
+except:
+    pass
