@@ -4,8 +4,8 @@ Copyright 2014--2020 Michael Hayes, UCECE
 
 """
 
-# The following are Lcapy expressions.  For example, f is an fExpr
-# object wrapping the fsym symbol.
+# The following are Lcapy expressions.  For example, f is an
+# FourierDomainExpression object wrapping the fsym symbol.
 from .fexpr import f
 from .texpr import t
 from .sexpr import s
@@ -15,3 +15,7 @@ from .omegaexpr import omega, omega0
 from .sym import pi, j, oo, inf, one
 
 jomega = j * omega
+jw = jomega
+
+jomega0 = j * omega0
+jw0 = jomega0
