@@ -1,5 +1,6 @@
-from lcapy import Circuit, R, C, L, V, I, v, exp, Heaviside, Vs, Vnoisy, Vt, It, sqrt, u, sympify, expr
-from lcapy import Zs, s, t
+from lcapy import *
+from lcapy.texpr import TimeDomainVoltage, TimeDomainCurrent
+from lcapy.sexpr import LaplaceDomainVoltage
 import unittest
 import sympy as sym
 
@@ -40,12 +41,14 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual2(
             a.admittance(1, 0), (R('R1') + C('C1')).Y, "Y incorrect for R1 + C1.")
         self.assertEqual2(a.Isc(1, 0), I(0).Isc, "Isc incorrect")
-        self.assertEqual(a.R1.Z, expr('R1'), "Z incorrect")
-        self.assertEqual(a.R1.R, expr('R1'), "R incorrect")
+
+        self.assertEqual(a.R1.Z, impedance('R1'), "Z incorrect")
+        self.assertEqual(a.R1.R, impedance('R1'), "R incorrect")
         self.assertEqual(a.R1.X, 0, "X incorrect")
-        self.assertEqual(a.C1.Y, expr('j * omega * C1'), "Y incorrect")
+        self.assertEqual(a.C1.Y, admittance('s * C1'), "Y incorrect")
         self.assertEqual(a.C1.G, 0, "G incorrect")
-        self.assertEqual(a.C1.B, expr('-omega * C1'), "B incorrect")        
+        # Use ac to force jomega form.
+        self.assertEqual(a.ac().C1.B, admittance('-omega * C1'), "B incorrect")        
         
 
     def test_VRC1(self):
@@ -232,7 +235,7 @@ class LcapyTester(unittest.TestCase):
         # This tests if symbols are converted to the defined ones.
         self.assertEqual2(a.L1.v, V(0).Voc.s.inverse_laplace(), 
                           "Incorrect time domain voltage")        
-        v = Vs('(V1+1)/s', dc=False).inverse_laplace()
+        v = LaplaceDomainVoltage('(V1+1)/s', dc=False).inverse_laplace()
         self.assertEqual2(a.R1.v, v, 
                           "Incorrect time domain voltage")        
         self.assertEqual(a.is_ivp, True, "Initial value problem incorrect")
@@ -326,10 +329,10 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(a.is_ac, True, "AC incorrect")
         self.assertEqual(a.is_causal, False, "Causal incorrect")
         self.assertEqual(a.is_time_domain, True, "Time domain incorrect")
-        self.assertEqual(a.V1.v, Vt('5*cos(t)'), "V1 voltage incorrect")
-        self.assertEqual(a.R1.v, Vt('5*cos(t)'), "R1 voltage incorrect")
-        self.assertEqual(a.V1.i, It('5*cos(t)'), "V1 current incorrect")
-        self.assertEqual(a.R1.i, It('5*cos(t)'), "R1 current incorrect")
+        self.assertEqual(a.V1.v, TimeDomainVoltage('5*cos(t)'), "V1 voltage incorrect")
+        self.assertEqual(a.R1.v, TimeDomainVoltage('5*cos(t)'), "R1 voltage incorrect")
+        self.assertEqual(a.V1.i, TimeDomainCurrent('5*cos(t)'), "V1 current incorrect")
+        self.assertEqual(a.R1.i, TimeDomainCurrent('5*cos(t)'), "R1 current incorrect")
 
         
     def test_VRL1_ac2(self):
@@ -346,8 +349,8 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(a.is_ac, True, "AC incorrect")
         self.assertEqual(a.is_causal, False, "Causal incorrect")
         self.assertEqual(a.is_time_domain, False, "Time domain incorrect")
-        self.assertEqual(a.V1.v, Vt('5*cos(t)'), "V1 voltage incorrect")
-        self.assertEqual(a.R1.i, It('(4*sin(t)+3*cos(t))/5'), "R1 current incorrect")
+        self.assertEqual(a.V1.v, TimeDomainVoltage('5*cos(t)'), "V1 voltage incorrect")
+        self.assertEqual(a.R1.i, TimeDomainCurrent('(4*sin(t)+3*cos(t))/5'), "R1 current incorrect")
         
 
     def test_VRC_ivp(self):
@@ -413,7 +416,7 @@ class LcapyTester(unittest.TestCase):
         # positive node to the negative node.
         self.assertEqual(a.R.I, -a.L.I, "R + L current different")
         self.assertEqual(a.R.V, a.L.V, "R + L voltage different")
-        self.assertEqual(a.L.I(s), sympify('i0 / (s + R / L)'), "L current wrong")
+        self.assertEqual(a.L.I(s), current('i0 / (s + R / L)'), "L current wrong")
 
     def test_RC_ivp(self):
         """Lcapy: check RC IVP"""
@@ -425,8 +428,7 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(a.is_ivp, True, "Initial value problem incorrect")
         self.assertEqual(a.R.I, -a.C.I, "R + C current different")
         self.assertEqual(a.R.V, a.C.V, "R + C voltage different")
-        self.assertEqual(a.C.I(s), sympify('-v0 / (s * R + 1 / C)'), "C current wrong")
-        
+        self.assertEqual(a.C.I(s), current('-v0 / (s * R + 1 / C)'), "C current wrong")
 
     def test_sub(self):
 
@@ -446,8 +448,8 @@ class LcapyTester(unittest.TestCase):
         I 1 0 dc
         R 2 0""")
 
-        self.assertEqual(a.R.v, expr('I * R'), "R voltage incorrect")
-        self.assertEqual(a.R.i, expr('I'), "R current incorrect")
+        self.assertEqual(a.R.v, voltage('I * R'), "R voltage incorrect")
+        self.assertEqual(a.R.i, current('I'), "R current incorrect")
 
     def test_IV_parallel(self):
 
@@ -456,8 +458,8 @@ class LcapyTester(unittest.TestCase):
         I 1 0 dc
         R 1 0""")
 
-        self.assertEqual(a.R.v, expr('V'), "R voltage incorrect")
-        self.assertEqual(a.R.i, expr('V / R'), "R current incorrect")        
+        self.assertEqual(a.R.v, voltage('V'), "R voltage incorrect")
+        self.assertEqual(a.R.i, current('V / R'), "R current incorrect")        
         
     def test_oneport(self):
 
@@ -524,7 +526,7 @@ class LcapyTester(unittest.TestCase):
         W 0_2 0_3; right
         K L1 L2 1; invisible""")
 
-        self.assertEqual(a.R1.v, expr('V1 * cos(omega_0 * t) * abs(omega_0) / omega_0'),
+        self.assertEqual(a.R1.v, voltage('V1 * cos(omega_0 * t) * abs(omega_0) / omega_0'),
                          "coupling incorrect")
         
 
@@ -556,7 +558,7 @@ class LcapyTester(unittest.TestCase):
         TF 2 0 1 0 k
         R 2 0""")        
 
-        self.assertEqual(a.impedance(1, 0), expr('R / k**2'), "incorrect impedance")
+        self.assertEqual(a.impedance(1, 0), impedance('R / k**2'), "incorrect impedance")
 
     def test_params(self):
 
@@ -626,3 +628,15 @@ class LcapyTester(unittest.TestCase):
 
         self.assertEqual(b.impedance(1, 2), a.impedance(1, 2), "simplify parallel")        
         
+
+
+    def test_VRL1_super(self):
+        """Lcapy: check VRL circuit
+
+        """
+        a = Circuit()
+        a.add('V1 1 0 {2 * u(t)}; down')
+        a.add('V2 1 4 4; right')
+        a.add('R1 4 2 2; right=2')
+        a.add('L1 2 3 2; down')
+        a.add('W 0 3; right')
