@@ -1,5 +1,10 @@
 from lcapy import *
-from lcapy.cexpr import cExpr
+from lcapy.cexpr import ConstantExpression, ConstantVoltage
+from lcapy.fexpr import FourierDomainExpression
+from lcapy.omegaexpr import AngularFourierDomainExpression
+from lcapy.texpr import TimeDomainExpression, TimeDomainVoltage
+from lcapy.sexpr import LaplaceDomainVoltage
+from lcapy.phasor import PhasorDomainVoltage, PhasorDomainExpression
 import unittest
 
 
@@ -15,54 +20,50 @@ class LcapyTester(unittest.TestCase):
             pprint(ans2)
             raise AssertionError(e)
 
-    def test_rmul(self):
-
-        self.assertEqual2((j * omega).__class__,  omegaExpr, "j * omega fail.")
-
     def test_Expr1(self):
         """Lcapy: check Expr
 
         """
-        a = Expr(3)
-        self.assertEqual2(a.N, Expr(3), "N incorrect.")
-        self.assertEqual2(a.D, Expr(1), "D incorrect.")
-        self.assertEqual2(a.real, Expr(3), "real incorrect.")
-        self.assertEqual2(a.imag, Expr(0), "imag incorrect.")
-        self.assertEqual2(a.magnitude, Expr(3), "magnitude incorrect.")
-        self.assertEqual2(a.phase, Expr(0), "phase incorrect.")
-        self.assertEqual2(a.phase_degrees, Expr(0), "phase incorrect.")
-        self.assertEqual2(a.sign, Expr(1), "sign incorrect.")
-        self.assertEqual2(-a.sign, Expr(-1), "sign incorrect.")
+        a = expr(3)
+        self.assertEqual2(a.N, expr(3), "N incorrect.")
+        self.assertEqual2(a.D, expr(1), "D incorrect.")
+        self.assertEqual2(a.real, expr(3), "real incorrect.")
+        self.assertEqual2(a.imag, expr(0), "imag incorrect.")
+        self.assertEqual2(a.magnitude, expr(3), "magnitude incorrect.")
+        self.assertEqual2(a.phase, expr(0), "phase incorrect.")
+        self.assertEqual2(a.phase_degrees, expr(0), "phase incorrect.")
+        self.assertEqual2(a.sign, expr(1), "sign incorrect.")
+        self.assertEqual2(-a.sign, expr(-1), "sign incorrect.")
 
-        a = Expr(-3)
-        self.assertEqual2(a.N, Expr(-3), "N incorrect.")
-        self.assertEqual2(a.D, Expr(1), "D incorrect.")
-        self.assertEqual2(a.real, Expr(-3), "real incorrect.")
-        self.assertEqual2(a.imag, Expr(0), "imag incorrect.")
-        self.assertEqual2(a.magnitude, Expr(3), "magnitude incorrect.")
-        self.assertEqual2(a.phase, Expr(pi), "phase incorrect.")
-        self.assertEqual2(a.phase_degrees, Expr(180), "phase incorrect.")
-        self.assertEqual2(a.sign, Expr(-1), "sign incorrect.")
-        self.assertEqual2(-a.sign, Expr(1), "sign incorrect.")        
+        a = expr(-3)
+        self.assertEqual2(a.N, expr(-3), "N incorrect.")
+        self.assertEqual2(a.D, expr(1), "D incorrect.")
+        self.assertEqual2(a.real, expr(-3), "real incorrect.")
+        self.assertEqual2(a.imag, expr(0), "imag incorrect.")
+        self.assertEqual2(a.magnitude, expr(3), "magnitude incorrect.")
+        self.assertEqual2(a.phase, expr(pi), "phase incorrect.")
+        self.assertEqual2(a.phase_degrees, expr(180), "phase incorrect.")
+        self.assertEqual2(a.sign, expr(-1), "sign incorrect.")
+        self.assertEqual2(-a.sign, expr(1), "sign incorrect.")        
 
-        a = Expr(3j)
-        self.assertEqual2(a.N, Expr(3j), "N incorrect.")
-        self.assertEqual2(a.D, Expr(1), "D incorrect.")
-        self.assertEqual2(a.real, Expr(0), "real incorrect.")
-        self.assertEqual2(a.imag, Expr(3), "imag incorrect.")        
-        self.assertEqual2(a.magnitude, Expr(3), "magnitude incorrect.")
-        self.assertEqual2(abs(a), Expr(3), "abs incorrect.")        
-        self.assertEqual2(a.phase, Expr(pi / 2), "phase incorrect.")
-        self.assertEqual2(a.phase_degrees, Expr(90), "phase incorrect.")
-        self.assertEqual2(a.sign, Expr(j), "sign incorrect.")
-        self.assertEqual2(-a.sign, Expr(-j), "sign incorrect.")        
+        a = expr(3j)
+        self.assertEqual2(a.N, expr(3j), "N incorrect.")
+        self.assertEqual2(a.D, expr(1), "D incorrect.")
+        self.assertEqual2(a.real, expr(0), "real incorrect.")
+        self.assertEqual2(a.imag, expr(3), "imag incorrect.")        
+        self.assertEqual2(a.magnitude, expr(3), "magnitude incorrect.")
+        self.assertEqual2(abs(a), expr(3), "abs incorrect.")        
+        self.assertEqual2(a.phase, expr(pi / 2), "phase incorrect.")
+        self.assertEqual2(a.phase_degrees, expr(90), "phase incorrect.")
+        self.assertEqual2(a.sign, expr(j), "sign incorrect.")
+        self.assertEqual2(-a.sign, expr(-j), "sign incorrect.")        
 
 
     def test_cExpr1(self):
         """Lcapy: check cExpr1
 
         """
-        a = cExpr('1')
+        a = ConstantExpression('1')
         self.assertEqual2(a.evaluate(), 1, "evaluate incorrect.")
         
         
@@ -70,9 +71,9 @@ class LcapyTester(unittest.TestCase):
         """Lcapy: check sExpr1
 
         """
-        a = sExpr('(s+2)/(s-2)')
-        self.assertEqual2(a.N, sExpr('s+2'), "N incorrect.")
-        self.assertEqual2(a.D, sExpr('s-2'), "D incorrect.")
+        a = LaplaceDomainExpression('(s+2)/(s-2)')
+        self.assertEqual2(a.N, LaplaceDomainExpression('s+2'), "N incorrect.")
+        self.assertEqual2(a.D, LaplaceDomainExpression('s-2'), "D incorrect.")
         self.assertEqual2(a.Ndegree, 1, "N degree incorrect.")
         self.assertEqual2(a.Ddegree, 1, "D degree incorrect.")
         self.assertEqual2(a.degree, 1, "Degree incorrect.")
@@ -89,7 +90,7 @@ class LcapyTester(unittest.TestCase):
 
         self.assertEqual(a.inverse_laplace(causal=True), 4 * exp(2 * t) * H(t) + DiracDelta(t), "inverse Laplace incorrect.")
 
-        aw = a(j * omega, causal=True)
+        aw = a.subs(j * omega, causal=True)
         self.assertEqual2(aw.real, (omega**2 - 4) / (omega**2 + 4), "real part incorrect.")
         self.assertEqual2(aw.imag, -4 * omega / (omega**2 + 4), "imag part incorrect.")
 
@@ -220,7 +221,7 @@ class LcapyTester(unittest.TestCase):
         """Lcapy: check sExpr7 (delay)
 
         """
-        a = sExpr('(s+1)*exp(-3*s)/((s+3)*(s+4))')
+        a = LaplaceDomainExpression('(s+1)*exp(-3*s)/((s+3)*(s+4))')
         self.assertEqual(a.inverse_laplace(causal=True), (3 * exp(-4 * t  + 12) - 2 * exp(-3 * t + 9)) * H(t- 3), "inverse Laplace incorrect.")
 
 
@@ -285,17 +286,6 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(a.evaluate(2), 0, "scalar evaluate incorrect.")
         self.assertEqual(a.laplace(), 1, "Laplace transform incorrect.")
 
-    def test_jomega(self):
-        """Lcapy: check jomega
-
-        """
-
-        a = sExpr('s+2')
-        b = a(j * omega, causal=True)
-
-        self.assertEqual2(b, j * omega + 2, "Substitution failed.")
-        self.assertEqual2(a.jomega, j * omega + 2, "jomega failed.")
-
     def test_subs1(self):
         """Lcapy: check subs
 
@@ -325,108 +315,31 @@ class LcapyTester(unittest.TestCase):
 
         """
 
-        c = cExpr(10)
-        self.assertEqual(type(10 + c), cExpr, "Not cExpr")
-        self.assertEqual(type(c + 10), cExpr, "Not cExpr")
-        self.assertEqual(type(cExpr(10) + c), cExpr, "Not cExpr")
-        self.assertEqual(type(c + cExpr(10)), cExpr, "Not cExpr")
+        c = ConstantExpression(10)
+        self.assertEqual(type(10 + c), ConstantExpression, "Not cExpr")
+        self.assertEqual(type(c + 10), ConstantExpression, "Not cExpr")
+        self.assertEqual(type(ConstantExpression(10) + c), ConstantExpression, "Not cExpr")
+        self.assertEqual(type(c + ConstantExpression(10)), ConstantExpression, "Not cExpr")
 
-        self.assertEqual(type(10 + s), sExpr, "Not sExpr")
-        self.assertEqual(type(s + 10), sExpr, "Not sExpr")
-        self.assertEqual(type(cExpr(10) + s), sExpr, "Not sExpr")
-        self.assertEqual(type(s + cExpr(10)), sExpr, "Not sExpr")
+        self.assertEqual(type(10 + s), LaplaceDomainExpression, "Not sExpr")
+        self.assertEqual(type(s + 10), LaplaceDomainExpression, "Not sExpr")
+        self.assertEqual(type(ConstantExpression(10) + s), LaplaceDomainExpression, "Not sExpr")
+        self.assertEqual(type(s + ConstantExpression(10)), LaplaceDomainExpression, "Not sExpr")
 
-        self.assertEqual(type(10 + t), tExpr, "Not tExpr")
-        self.assertEqual(type(t + 10), tExpr, "Not tExpr")
-        self.assertEqual(type(cExpr(10) + t), tExpr, "Not tExpr")
-        self.assertEqual(type(t + cExpr(10)), tExpr, "Not tExpr")
+        self.assertEqual(type(10 + t), TimeDomainExpression, "Not tExpr")
+        self.assertEqual(type(t + 10), TimeDomainExpression, "Not tExpr")
+        self.assertEqual(type(ConstantExpression(10) + t), TimeDomainExpression, "Not tExpr")
+        self.assertEqual(type(t + ConstantExpression(10)), TimeDomainExpression, "Not tExpr")
 
-        v = Vs(10)
-        self.assertEqual(type(10 + v), Vs, "Not Vs")
-        self.assertEqual(type(v + 10), Vs, "Not Vs")
-        self.assertEqual(type(sExpr(10) + v), Vs, "Not Vs")
-        self.assertEqual(type(v + sExpr(10)), Vs, "Not Vs")
+        v = LaplaceDomainVoltage(10)
+        self.assertEqual(type(10 + v), LaplaceDomainVoltage, "Not Vs")
+        self.assertEqual(type(v + 10), LaplaceDomainVoltage, "Not Vs")
+        self.assertEqual(type(LaplaceDomainExpression(10) + v), LaplaceDomainVoltage, "Not Vs")
+        self.assertEqual(type(v + LaplaceDomainExpression(10)), LaplaceDomainVoltage, "Not Vs")
 
-        self.assertEqual(type(omega * t), tExpr, "Not tExpr")
-        self.assertEqual(type(t * omega), tExpr, "Not tExpr")                
+        self.assertEqual(type(omega * t), TimeDomainExpression, "Not tExpr")
+        self.assertEqual(type(t * omega), TimeDomainExpression, "Not tExpr")                
 
-    def test_Vs(self):
-        """Lcapy: check Vs
-
-        """
-
-        self.assertEqual(type(Vs(5) * 5), Vs, "Not Vs")
-        self.assertEqual(type(Vs(5) * Vs(4)), Vs, "Not Vs")
-        self.assertEqual(type(Vs(5) * Ys(4)), Is, "Not Is")
-        self.assertEqual(type(Vs(5) / Zs(4)), Is, "Not Is")
-
-    def test_Vt(self):
-        """Lcapy: check Vt
-
-        """
-
-        self.assertEqual(type(Vt(5) * 5), Vt, "Not Vt")
-        self.assertEqual(type(Vt(5) * Vt(4)), Vt, "Not Vt")
-
-    def test_Vf(self):
-        """Lcapy: check Vf
-
-        """
-
-        self.assertEqual(type(Vf(5) * 5), Vf, "Not Vf")
-        self.assertEqual(type(Vf(5) * Vf(4)), Vf, "Not Vf")
-        self.assertEqual(type(Vf(5) * Yf(4)), If, "Not If")
-        self.assertEqual(type(Vf(5) / Zf(4)), If, "Not If")
-
-    def test_Vomega(self):
-        """Lcapy: check Vomega
-
-        """
-
-        self.assertEqual(type(Vomega(5) * 5), Vomega, "Not Vomega")
-        self.assertEqual(type(Vomega(5) * Vomega(4)), Vomega, "Not Vomega")
-        self.assertEqual(type(Vomega(5) * Yomega(4)), Iomega, "Not Iomega")
-        self.assertEqual(type(Vomega(5) / Zomega(4)), Iomega, "Not Iomega")
-
-    def test_Is(self):
-        """Lcapy: check Is
-
-        """
-
-        self.assertEqual(type(Is(5) * 5), Is, "Not Is")
-        self.assertEqual(type(Is(5) * Is(4)), Is, "Not Is")
-        self.assertEqual(type(Is(5) * Zs(4)), Vs, "Not Vs")
-        self.assertEqual(type(Is(5) / Ys(4)), Vs, "Not Vs")
-
-    def test_It(self):
-        """Lcapy: check It
-
-        """
-
-        self.assertEqual(type(It(5) * 5), It, "Not It")
-        self.assertEqual(type(It(5) * It(4)), It, "Not It")
-
-    def test_If(self):
-        """Lcapy: check If
-
-        """
-
-        self.assertEqual(type(If(5) * 5), If, "Not If")
-        self.assertEqual(type(If(5) * If(4)), If, "Not If")
-        self.assertEqual(type(If(5) / Yf(4)), Vf, "Not Vf")
-        self.assertEqual(type(If(5) * Zf(4)), Vf, "Not Vf")
-
-    def test_Iomega(self):
-        """Lcapy: check Iomega
-
-        """
-
-        self.assertEqual(type(Iomega(5) * 5), Iomega, "Not Iomega")
-        self.assertEqual(type(Iomega(5) * Iomega(4)), Iomega, "Not Iomega")
-        self.assertEqual(type(Iomega(5) / Yomega(4)), Vomega, "Not Vomega")
-        self.assertEqual(type(Iomega(5) * Zomega(4)), Vomega, "Not Vomega")
-
-        
     def test_evaluate(self):
         """Lcapy: check evaluate
 
@@ -448,8 +361,8 @@ class LcapyTester(unittest.TestCase):
 
     def test_rms(self):
 
-        self.assertEqual(Vconst(2).rms(), Vt(2))
-        self.assertEqual(Vphasor(2).rms(), Vt(1))
+        self.assertEqual(ConstantVoltage(2).rms(), TimeDomainVoltage(2))
+        self.assertEqual(PhasorDomainVoltage(2).rms(), TimeDomainVoltage(1))
 
     def test_assumptions(self):
         """Lcapy: check assumptions
@@ -473,7 +386,9 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual((Heaviside(t).laplace() *
                           DiracDelta(t).laplace()).is_causal, True)
         self.assertEqual((Heaviside(t).laplace() *
-                          cos(t).laplace()).is_causal, True)
+                          cos(t).laplace()).is_causal, False)
+        self.assertEqual((Heaviside(t).laplace() *
+                          cos(t).laplace()).is_ac, True)        
         self.assertEqual(Heaviside(2 * t).is_causal, True, "Heaviside(2 * t).is_causal")
         self.assertEqual(Heaviside(t - 1).is_causal, True, "Heaviside(t - 1).is_causal")
         self.assertEqual(Heaviside(t + 1).is_causal, False, "Heaviside(t + 1).is_causal")
@@ -493,7 +408,7 @@ class LcapyTester(unittest.TestCase):
     def test_expr(self):
 
         a = expr('3 * exp(-t) * t * a')
-        self.assertEqual(isinstance(a, tExpr), True, "tExpr")        
+        self.assertEqual(isinstance(a, TimeDomainExpression), True, "tExpr")        
         self.assertEqual(a.has(3), True, "has(3)")
         self.assertEqual(a.has(4), False, "has(4)")
         self.assertEqual(a.has(t), True, "has(t)")
@@ -504,12 +419,17 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(a.is_constant, False, "is_constant")                
 
         a = expr('3')
-        self.assertEqual(a.is_constant, True, "3 is_constant")        
+        self.assertEqual(a.is_constant, True, "3 is_constant")
+        self.assertEqual(a.is_unchanging, True, "3 is_unchanging")
 
-        self.assertEqual(isinstance(expr(t), tExpr), True, "tExpr")
-        self.assertEqual(isinstance(expr(s), sExpr), True, "sExpr")
-        self.assertEqual(isinstance(expr(f), fExpr), True, "fExpr")
-        self.assertEqual(isinstance(expr(omega), omegaExpr), True, "omegaExpr")
+        a = expr('3 * x')
+        self.assertEqual(a.is_constant, False, "3 * x is_constant")
+        self.assertEqual(a.is_unchanging, True, "3 * x is_unchanging")        
+
+        self.assertEqual(isinstance(expr(t), TimeDomainExpression), True, "tExpr")
+        self.assertEqual(isinstance(expr(s), LaplaceDomainExpression), True, "sExpr")
+        self.assertEqual(isinstance(expr(f), FourierDomainExpression), True, "fExpr")
+        self.assertEqual(isinstance(expr(omega), AngularFourierDomainExpression), True, "omegaExpr")
         
         self.assertEqual(isinstance(symbol('c'), Expr), True, "symbol")
 
@@ -522,33 +442,6 @@ class LcapyTester(unittest.TestCase):
         if expr('omega') is not omega:
             raise AssertionError('Not omega')
 
-    def test_comparison(self):
-
-        a = expr('3')
-        b = expr('4')
-        self.assertEqual(a < b, True, "a < b")
-        self.assertEqual(a > b, False, "a > b")
-        self.assertEqual(a <= b, True, "a <= b")
-        self.assertEqual(a >= b, False, "a >= b")        
-        self.assertEqual(a == b, False, "a == b")
-        self.assertEqual(a != b, True, "a != b")        
-
-    def test_rdiv(self):
-
-        self.assertEqual(3 / expr('3'), 1, "3 / expr('3')")
-
-    def test_div(self):
-
-        self.assertEqual(expr('3') / 3, 1, "expr('3') / 3")        
-        
-    def test_rsub(self):
-
-        self.assertEqual(3 - expr('3'), 0, "3 - expr('3')")
-
-    def test_sub(self):
-
-        self.assertEqual(expr('3') - 3, 0, "expr('3') - 3")
-
     def test_parallel(self):
 
         self.assertEqual(expr('4').parallel(expr('4')), 2, "parallel")
@@ -580,7 +473,6 @@ class LcapyTester(unittest.TestCase):
 
         self.assertEqual(t.integrate((t, 0, t)), t**2 / 2, "integrate t")
         
-
     def test_partfrac(self):
 
         H = expr('F(s)') / (s**2 + 3 * s + 6)
@@ -628,7 +520,6 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(F.partfrac(), F,  "undef delay sum partfrac")
         self.assertEqual(F.partfrac(True), F,  "undef delay sum partfrac")
         
-
     def test_mixedfrac(self):
 
         H = expr('F(s)') / (s**2 + 3 * s + 6)
@@ -688,7 +579,6 @@ class LcapyTester(unittest.TestCase):
         x = X(t, condition=True)
         self.assertEqual(x.strip_condition(), exp(-t), "remove condition if causal")
         
-        
     def test_val(self):
 
         a = expr('3 / 4')
@@ -713,7 +603,6 @@ class LcapyTester(unittest.TestCase):
 
         self.assertEqual(e.cval, a, 'cval')
 
-        
     def test_fval(self):
 
         a = 3
@@ -723,14 +612,33 @@ class LcapyTester(unittest.TestCase):
 
     def test_phasor(self):
 
-        a = Phasor(3, omega=7)
+        a = PhasorDomainExpression(3, omega=7)
         self.assertEqual(a.omega, 7, 'omega')
         self.assertEqual(a.phase, 0, 'phase')
         self.assertEqual(a.magnitude, 3, 'magnitude')        
         
-        a = Phasor(-3, omega=7)
+        a = PhasorDomainExpression(-3, omega=7)
         self.assertEqual(a.phase, pi, 'phase')                
         self.assertEqual(a.magnitude, 3, 'magnitude')
 
-        a = Phasor(-3 + 4j, omega=7)
+        a = PhasorDomainExpression(-3 + 4j, omega=7)
         self.assertEqual(a.magnitude, 5, 'magnitude')                        
+
+        v = voltage('cos(5 * t)')
+        i = current('cos(5 * t)')
+        V = v.phasor()
+        I = i.phasor()
+
+        Z = V / I
+        self.assertEqual(Z.is_impedance, True, 'impedance')
+        self.assertEqual(Z.omega, 5, 'omega')
+        
+    def test_equality(self):
+
+        e = Eq(expr('x(t)'), cos(3 * t))
+        self.assertEqual(e.is_ac, True, 'is_ac')
+        self.assertEqual(e.is_dc, False, 'is_dc')
+        self.assertEqual(e.is_causal, False, 'is_causal')
+        
+               
+        
