@@ -2,15 +2,18 @@
 This module supports simple linear three-port networks.  It is
 experimental and needs a rethink.
 
-Copyright 2014--2019 Michael Hayes, UCECE
+Copyright 2014--2020 Michael Hayes, UCECE
 
 """
 
 from __future__ import division
 from warnings import warn
 import sympy as sym
-from .sexpr import Vs, Hs, Is, VsVector, IsVector, YsVector, ZsVector
-from .cexpr import cExpr
+from .sexpr import LaplaceDomainVoltage, LaplaceDomainTransferFunction
+from .sexpr import LaplaceDomainCurrent
+from .smatrix import LaplaceDomainVoltageMatrix, LaplaceDomainCurrentMatrix
+from .smatrix import LaplaceDomainImpedanceMatrix, LaplaceDomainAdmittanceMatrix
+from .cexpr import ConstantExpression
 from .oneport import OnePort
 from .twoport import YMatrix, ZMatrix, TwoPortZModel, Series, TwoPort
 
@@ -90,13 +93,16 @@ class ThreePort(object):
 
     """
 
-    def __init__(self, Z, Vz=VsVector((0, 0, 0))):
+    def __init__(self, Z, Vz=None):
+
+        if Vz is None:
+            Vz = LaplaceDomainVoltageMatrix((0, 0, 0))
 
         if not isinstance(Z, ZMatrix3):
             raise ValueError('Z not ZMatrix3')
 
-        if not isinstance(Vz, VsVector):
-            raise ValueError('Vz not VsVector')
+        if not isinstance(Vz, LaplaceDomainVoltageMatrix):
+            raise ValueError('Vz not LaplaceDomainVoltageMatrix')
 
         self._M = Z
         self._Vz = Vz
@@ -114,7 +120,7 @@ class ThreePort(object):
         Y = self.Y
         Voc = self.Voc
 
-        Isc = IsVector([Voc[m] * Y[m, m] for m in range(len(Voc))])
+        Isc = LaplaceDomainCurrentMatrix([Voc[m] * Y[m, m] for m in range(len(Voc))])
         return Isc
 
     @property
@@ -131,25 +137,25 @@ class ThreePort(object):
     def Yoc(self):
         """Return admittance vector with ports open circuit"""
         Z = self.Z
-        return YsVector([1 / Z[m, m] for m in range(Z.shape[0])])
+        return LaplaceDomainAdmittanceMatrix([1 / Z[m, m] for m in range(Z.shape[0])])
 
     @property
     def Ysc(self):
         """Return admittance vector with ports short circuit"""
         Y = self.Y
-        return YsVector([Y[m, m] for m in range(Y.shape[0])])
+        return LaplaceDomainAdmittanceMatrix([Y[m, m] for m in range(Y.shape[0])])
 
     @property
     def Zoc(self):
         """Return impedance vector with ports open circuit"""
         Z = self.Z
-        return ZsVector([Z[m, m] for m in range(Z.shape[0])])
+        return LaplaceDomainImpedanceMatrix([Z[m, m] for m in range(Z.shape[0])])
 
     @property
     def Zsc(self):
         """Return impedance vector with ports short circuit"""
         Y = self.Y
-        return ZsVector([1 / Y[m, m] for m in range(Y.shape[0])])
+        return LaplaceDomainImpedanceMatrix([1 / Y[m, m] for m in range(Y.shape[0])])
 
     def _portcheck(self, port):
 
@@ -166,7 +172,7 @@ class ThreePort(object):
         p1 = inport - 1
         p2 = outport - 1
 
-        return Hs(self.Z[p2, p1] / self.Z[p1, p1])
+        return LaplaceDomainTransferFunction(self.Z[p2, p1] / self.Z[p1, p1])
 
     def Igain(self, inport=1, outport=2):
         """Return voltage gain for specified ports with internal
@@ -178,7 +184,7 @@ class ThreePort(object):
         p1 = inport - 1
         p2 = outport - 1
 
-        return Hs(self.Y[p2, p1] / self.Y[p1, p1])
+        return LaplaceDomainTransferFunction(self.Y[p2, p1] / self.Y[p1, p1])
 
     def Vresponse(self, V, inport=1, outport=2):
         """Return voltage response for specified applied voltage and
@@ -191,7 +197,7 @@ class ThreePort(object):
         p2 = outport - 1
 
         H = self.Z[p2, p1] / self.Z[p1, p1]
-        return Vs(self.Voc[p2] + (V - self.Voc[p1]) * H)
+        return LaplaceDomainVoltage(self.Voc[p2] + (V - self.Voc[p1]) * H)
 
     def Iresponse(self, I, inport=1, outport=2):
         """Return current response for specified current voltage and
@@ -206,7 +212,7 @@ class ThreePort(object):
         Y = self.Y
         Isc = self.Isc
 
-        return Is(Isc[p2] + (I - Isc[p1]) * Y[p2, p1] / Y[p1, p1])
+        return LaplaceDomainCurrent(Isc[p2] + (I - Isc[p1]) * Y[p2, p1] / Y[p1, p1])
 
     def attach_parallel(self, OP, port=2):
         """Attach one-port in parallel to specified port"""
@@ -223,7 +229,7 @@ class ThreePort(object):
         Isc = self.Isc
         Isc[p] += OP.Isc
         Z = Y.Z
-        Voc = VsVector([Vs(Isc[m] * Z[m, m]) for m in range(len(Isc))])
+        Voc = LaplaceDomainVoltageMatrix([LaplaceDomainVoltage(Isc[m] * Z[m, m]) for m in range(len(Isc))])
         return ThreePort(Z, Voc)
 
     def bridge(self, OP, inport=1, outport=2):
@@ -252,7 +258,7 @@ class ThreePort(object):
         Isc[p1] -= OP.Isc
         Isc[p2] += OP.Isc
         Z = Y.Z
-        Voc = VsVector([Vs(Isc[m] * Z[m, m]) for m in range(len(Isc))])
+        Voc = LaplaceDomainVoltageMatrix([LaplaceDomainVoltage(Isc[m] * Z[m, m]) for m in range(len(Isc))])
         return ThreePort(Y.Z, Voc)
 
     def parallel(self, MP, port=None):
@@ -271,7 +277,7 @@ class ThreePort(object):
         Y = self.Y + MP.Y
         Isc = self.Isc + MP.Isc
         Z = Y.Z
-        Voc = VsVector([Vs(Isc[m] * Z[m, m]) for m in range(len(Isc))])
+        Voc = LaplaceDomainVoltageMatrix([LaplaceDomainVoltage(Isc[m] * Z[m, m]) for m in range(len(Isc))])
         return ThreePort(Z, Voc)
 
     def series(self, MP, port=None):
@@ -313,7 +319,7 @@ class ThreePort(object):
         Voc = self.Voc.copy()
         Voc.row_del(port - 1)
 
-        return TwoPortZModel(Y.Z, Vs(Voc[0]), Vs(Voc[1]))
+        return TwoPortZModel(Y.Z, LaplaceDomainVoltage(Voc[0]), LaplaceDomainVoltage(Voc[1]))
 
     def open_circuit(self, port=2):
         """Apply a open-circuit to specified port and return a
@@ -328,7 +334,7 @@ class ThreePort(object):
         Voc = self.Voc.copy()
         Voc.row_del(port - 1)
 
-        return TwoPortZModel(Z, Vs(Voc[0]), Vs(Voc[1]))
+        return TwoPortZModel(Z, LaplaceDomainVoltage(Voc[0]), LaplaceDomainVoltage(Voc[1]))
 
 
 class Opamp(ThreePort):
@@ -360,7 +366,7 @@ class Opamp(ThreePort):
 
         # If Ro=0, then Z matrix singular.
 
-        Rd, Ro, A, Rp, Rm = [cExpr(arg) for arg in (Rd, Ro, A, Rp, Rm)]
+        Rd, Ro, A, Rp, Rm = [ConstantExpression(arg) for arg in (Rd, Ro, A, Rp, Rm)]
 
         Ra = Rp * (Rd + Rm) / (Rp + Rd + Rm)
         Rb = Rm * (Rd + Rp) / (Rp + Rd + Rm)
