@@ -1,35 +1,36 @@
-"""This module provides the zExpr class to represent z-domain expressions.
+"""This module provides the ZDomainExpression class to represent z-domain expressions.
 
-Copyright 2020 Michael Hayes, UCECE
+Copyright 2020--2021 Michael Hayes, UCECE
 
 """
 
 from __future__ import division
+from .domains import ZDomain
 from .ztransform import inverse_ztransform
 from .sym import j, pi
 from .dsym import nsym, ksym, zsym, dt
 from .vector import Vector
 from .ratfun import _zp2tf, Ratfun
-from .dexpr import dExpr
+from .dexpr import DiscreteExpression
 from .expr import symbol, expr, ExprDict
 from .functions import sqrt, exp
 import numpy as np
 from sympy import Eq, div, limit, oo, Sum
 
 
-__all__ = ('Hz', 'Iz', 'Vz', 'Yz', 'Zz')
+__all__ = ('zexpr', )
 
-class zExpr(dExpr):
+class ZDomainExpression(ZDomain, DiscreteExpression):
     """z-domain expression or symbol."""
 
     var = zsym
 
+
     def __init__(self, val, **assumptions):
 
         check = assumptions.pop('check', True)
 
-        super(zExpr, self).__init__(val, **assumptions)
-        self._ztransform_conjugate_class = nExpr
+        super(ZDomainExpression, self).__init__(val, **assumptions)
 
         expr = self.expr
         if check and expr.find(nsym) != set() and not expr.has(Sum):
@@ -39,6 +40,9 @@ class zExpr(dExpr):
             raise ValueError(
                 'z-domain expression %s cannot depend on k' % expr)
 
+    def as_expr(self):
+        return ZDomainExpression(self)
+
     def ndifferentiate(self):
         """First order difference in n-domain."""
 
@@ -71,16 +75,9 @@ class zExpr(dExpr):
 
         """
 
-        if assumptions == {}:
-            assumptions = self.assumptions.copy()
-
+        assumptions = self.assumptions.merge(**assumptions)
         result = inverse_ztransform(self.expr, self.var, nsym, **assumptions)
-
-        if hasattr(self, '_ztransform_conjugate_class'):
-            result = self._ztransform_conjugate_class(result)
-        else:
-            result = nexpr(result)
-        return result
+        return self.change(result, domain='discrete time', **assumptions)
 
     def IZT(self, **assumptions):
         return self.inverse_ztransform(**assumptions)
@@ -138,7 +135,7 @@ class zExpr(dExpr):
 
         # Evaluate transient response.
         th = np.arange(N) * dt - dt
-        h = zExpr(expr).transient_response(th)
+        h = ZDomainExpression(expr).transient_response(th)
 
         print('Convolving...')
         ty = t
@@ -172,7 +169,7 @@ class zExpr(dExpr):
 
     def evaluate(self, svector=None):
 
-        return super(zExpr, self).evaluate(svector)
+        return super(ZDomainExpression, self).evaluate(svector)
 
     def plot(self, t=None, **kwargs):
         """Plot pole-zero map."""
@@ -214,8 +211,8 @@ class zExpr(dExpr):
         invz = expr('invz')
         H = R.replace(z, 1 / invz).factor()
         r = Ratfun(H, invz.expr)
-        B = zExpr(r.N).replace(invz, 1 / z)
-        A = zExpr(r.D).replace(invz, 1 / z)
+        B = ZDomainExpression(r.N).replace(invz, 1 / z)
+        A = ZDomainExpression(r.D).replace(invz, 1 / z)
 
         C1, R1 = A.term_const()
         if C1.is_negative:
@@ -252,103 +249,21 @@ class zExpr(dExpr):
         else:
             raise ValueError('Unhandled form ' + form)    
 
-        return nExpr(Eq(lhs.expr, rhs.expr))        
+        return DiscreteTimeDomainExpression(Eq(lhs.expr, rhs.expr))        
         
     
-# Perhaps use a factory to create the following classes?
-
-class Zz(zExpr):
-
-    """z-domain impedance value."""
-
-    quantity = 'Impedance'
-    units = 'ohms'
-
-    def __init__(self, val, causal=True, **assumptions):
-
-        super(Zz, self).__init__(val, causal=causal, **assumptions)
-        self._ztransform_conjugate_class = Zn
-
-
-class Yz(zExpr):
-
-    """z-domain admittance value."""
-
-    quantity = 'Admittance'
-    units = 'siemens'
-
-    def __init__(self, val, causal=True, **assumptions):
-
-        super(Yz, self).__init__(val, causal=causal, **assumptions)
-        self._ztransform_conjugate_class = Yn
-
-
-class Vz(zExpr):
-
-    """z-domain voltage (units V s / radian)."""
-
-    quantity = 'z-Voltage'
-    units = 'V/Hz'
-
-    def __init__(self, val, **assumptions):
-
-        super(Vz, self).__init__(val, **assumptions)
-        self._ztransform_conjugate_class = Vn
-
-
-class Iz(zExpr):
-
-    """z-domain current (units A s / radian)."""
-
-    quantity = 'z-Current'
-    units = 'A/Hz'
-
-    def __init__(self, val, **assumptions):
-
-        super(Iz, self).__init__(val, **assumptions)
-        self._ztransform_conjugate_class = In
-
-
-class Hz(zExpr):
-
-    """z-domain ratio"""
-
-    quantity = 'z-ratio'
-    units = ''
-
-    def __init__(self, val, **assumptions):
-
-        super(Hz, self).__init__(val, **assumptions)
-        self._ztransform_conjugate_class = Hn
-
-        
-class VzVector(Vector):
-
-    _typewrap = Vz
-
-
-class IzVector(Vector):
-
-    _typewrap = Iz
-
-
-class YzVector(Vector):
-
-    _typewrap = Yz
-
-
-class ZzVector(Vector):
-
-    _typewrap = Zz
-
-
 def zexpr(arg, **assumptions):
-    """Create zExpr object.  If `arg` is zsym return z"""
+    """Create ZDomainExpression object.  If `arg` is zsym return z"""
 
     if arg is zsym:
         return z
-    return zExpr(arg, **assumptions)
+    return ZDomainExpression(arg, **assumptions)
 
 
-from .nexpr import Hn, In, Vn, Yn, Zn, nExpr, nexpr
-z = zExpr('z')
+from .expressionclasses import expressionclasses
+
+classes = expressionclasses.make(ZDomainExpression)
+expressionclasses.add('Z', classes)
+
+from .nexpr import DiscreteTimeDomainExpression, nexpr
+z = ZDomainExpression('z')
